@@ -54,9 +54,10 @@ mod tests {
             Chain, ChainSource, Collection, Folder, Profile, Recipe,
             RecipeBody, RecipeNode,
         },
-        http::{content_type::ContentType, HttpMethod},
-        test_util::{by_id, temp_dir, Factory, TempDir},
+        http::{HttpMethod, content_type::ContentType},
+        test_util::by_id,
     };
+    use slumber_util::{Factory, TempDir, temp_dir};
     use std::{env, fs};
 
     /// Test creating a new collection file, specifying the path in various ways
