@@ -1,6 +1,3 @@
-#![forbid(unsafe_code)]
-#![deny(clippy::all)]
-
 //! Command line interface for Slumber.
 //!
 //! **This crate is not semver compliant**. The version is locked to the root
@@ -11,12 +8,13 @@ mod commands;
 mod completions;
 
 use crate::commands::{
-    collections::CollectionsCommand, generate::GenerateCommand,
+    collections::CollectionsCommand, db::DbCommand, generate::GenerateCommand,
     history::HistoryCommand, import::ImportCommand, new::NewCommand,
     request::RequestCommand, show::ShowCommand,
 };
 use clap::{CommandFactory, Parser};
 use clap_complete::CompleteEnv;
+use slumber_core::collection::CollectionFile;
 use std::{path::PathBuf, process::ExitCode};
 
 const COMMAND_NAME: &str = "slumber";
@@ -25,7 +23,7 @@ const COMMAND_NAME: &str = "slumber";
 ///
 /// If subcommand is omitted, start the TUI.
 ///
-/// https://slumber.lucaspickering.me/book/
+/// <https://slumber.lucaspickering.me/book/>
 #[derive(Debug, Parser)]
 #[clap(author, version, about, name = COMMAND_NAME)]
 pub struct Args {
@@ -60,10 +58,19 @@ pub struct GlobalArgs {
     pub file: Option<PathBuf>,
 }
 
+impl GlobalArgs {
+    /// Get the path to the active collection file. Return an error if there is
+    /// no collection file present, or if the user specified an invalid file.
+    fn collection_file(&self) -> anyhow::Result<CollectionFile> {
+        CollectionFile::new(self.file.clone())
+    }
+}
+
 /// A CLI subcommand
 #[derive(Clone, Debug, clap::Subcommand)]
 pub enum CliCommand {
     Collections(CollectionsCommand),
+    Db(DbCommand),
     Generate(GenerateCommand),
     History(HistoryCommand),
     Import(ImportCommand),
@@ -77,6 +84,7 @@ impl CliCommand {
     pub async fn execute(self, global: GlobalArgs) -> anyhow::Result<ExitCode> {
         match self {
             Self::Collections(command) => command.execute(global).await,
+            Self::Db(command) => command.execute(global).await,
             Self::Generate(command) => command.execute(global).await,
             Self::History(command) => command.execute(global).await,
             Self::Import(command) => command.execute(global).await,
