@@ -5,7 +5,7 @@ use crossterm::event::{KeyCode, KeyEvent, KeyModifiers, MediaKeyCode};
 use derive_more::Display;
 use itertools::Itertools;
 use serde::{Deserialize, Serialize};
-use slumber_core::util::Mapping;
+use slumber_util::Mapping;
 use std::{
     borrow::Cow,
     fmt::{self, Debug},
@@ -141,6 +141,8 @@ pub enum Action {
     Toggle,
     /// Close the current modal/dialog/etc. OR cancel a request
     Cancel,
+    /// Delete the selected object (e.g. a request)
+    Delete,
     /// Trigger the workflow to provide a temporary override for a recipe value
     /// (body/param/etc.)
     Edit,
@@ -227,7 +229,7 @@ impl Display for InputBinding {
             if i > 0 {
                 write!(f, ",")?;
             }
-            write!(f, "{}", combo)?;
+            write!(f, "{combo}")?;
         }
         Ok(())
     }
@@ -298,7 +300,8 @@ impl Display for KeyCombination {
             KeyCode::Right => write!(f, "→"),
             KeyCode::Esc => write!(f, "<esc>"),
             KeyCode::Enter => write!(f, "<enter>"),
-            KeyCode::F(num) => write!(f, "F{}", num),
+            KeyCode::Delete => write!(f, "<del>"),
+            KeyCode::F(num) => write!(f, "F{num}"),
             KeyCode::Char(' ') => write!(f, "<space>"),
             KeyCode::Char(c) => write!(f, "{c}"),
             // Punting on everything else until we need it
@@ -418,8 +421,8 @@ mod tests {
     use super::*;
     use crossterm::event::{KeyEventKind, KeyEventState, MediaKeyCode};
     use rstest::rstest;
-    use serde_test::{assert_de_tokens, assert_de_tokens_error, Token};
-    use slumber_core::assert_err;
+    use serde_test::{Token, assert_de_tokens, assert_de_tokens_error};
+    use slumber_util::assert_err;
 
     #[rstest]
     #[case::whitespace_stripped(" w ", KeyCode::Char('w'))]
