@@ -5,16 +5,16 @@ use crate::{
         cereal,
         recipe_tree::{RecipeNode, RecipeTree},
     },
-    http::{content_type::ContentType, query::Query, HttpMethod},
+    http::{HttpMethod, content_type::ContentType, query::Query},
     template::{Identifier, Template},
-    util::{parse_yaml, ResultTraced},
 };
 use anyhow::Context;
-use derive_more::{Deref, Display, From, FromStr};
+use derive_more::{Deref, Display, From, FromStr, Into};
 use indexmap::IndexMap;
 use mime::Mime;
 use reqwest::header;
 use serde::{Deserialize, Serialize};
+use slumber_util::{ResultTraced, parse_yaml};
 use std::{fs::File, path::PathBuf, time::Duration};
 use tracing::info;
 
@@ -40,6 +40,7 @@ pub struct Collection {
     /// ignore anything that starts with `.` (recursively) but that
     /// requires a custom serde impl for each type, or changes to the macro
     #[serde(default, skip_serializing, rename = ".ignore")]
+    #[expect(clippy::pub_underscore_fields)]
     pub _ignore: serde::de::IgnoredAny,
 }
 
@@ -89,8 +90,8 @@ impl Profile {
 }
 
 #[cfg(any(test, feature = "test"))]
-impl crate::test_util::Factory for Profile {
-    fn factory(_: ()) -> Self {
+impl slumber_util::Factory for Profile {
+    fn factory((): ()) -> Self {
         Self {
             id: ProfileId::factory(()),
             name: None,
@@ -109,6 +110,7 @@ impl crate::test_util::Factory for Profile {
     Eq,
     From,
     Hash,
+    Into,
     PartialEq,
     Serialize,
     Deserialize,
@@ -125,8 +127,8 @@ impl From<&str> for ProfileId {
 }
 
 #[cfg(any(test, feature = "test"))]
-impl crate::test_util::Factory for ProfileId {
-    fn factory(_: ()) -> Self {
+impl slumber_util::Factory for ProfileId {
+    fn factory((): ()) -> Self {
         uuid::Uuid::new_v4().to_string().into()
     }
 }
@@ -156,8 +158,8 @@ impl Folder {
 }
 
 #[cfg(any(test, feature = "test"))]
-impl crate::test_util::Factory for Folder {
-    fn factory(_: ()) -> Self {
+impl slumber_util::Factory for Folder {
+    fn factory((): ()) -> Self {
         Self {
             id: RecipeId::factory(()),
             name: None,
@@ -170,12 +172,15 @@ impl crate::test_util::Factory for Folder {
 /// order to distinguish it from a single instance of an HTTP request. And it's
 /// not called `RequestTemplate` because the word "template" has a specific
 /// meaning related to string interpolation.
+#[expect(clippy::unsafe_derive_deserialize)]
 #[derive(Debug, Serialize, Deserialize)]
 #[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
 #[serde(deny_unknown_fields)]
 pub struct Recipe {
     #[serde(skip)] // This will be auto-populated from the map key
     pub id: RecipeId,
+    #[serde(default = "cereal::persist_default")]
+    pub persist: bool,
     pub name: Option<String>,
     /// *Not* a template string because the usefulness doesn't justify the
     /// complexity. This gives the user an immediate error if the method is
@@ -211,10 +216,11 @@ impl Recipe {
 }
 
 #[cfg(any(test, feature = "test"))]
-impl crate::test_util::Factory for Recipe {
-    fn factory(_: ()) -> Self {
+impl slumber_util::Factory for Recipe {
+    fn factory((): ()) -> Self {
         Self {
             id: RecipeId::factory(()),
+            persist: true,
             name: None,
             method: HttpMethod::Get,
             url: "http://localhost/url".into(),
@@ -228,7 +234,7 @@ impl crate::test_util::Factory for Recipe {
 
 /// Create recipe with a fixed ID
 #[cfg(any(test, feature = "test"))]
-impl crate::test_util::Factory<&str> for Recipe {
+impl slumber_util::Factory<&str> for Recipe {
     fn factory(id: &str) -> Self {
         Self {
             id: id.into(),
@@ -246,6 +252,7 @@ impl crate::test_util::Factory<&str> for Recipe {
     Eq,
     From,
     Hash,
+    Into,
     PartialEq,
     Serialize,
     Deserialize,
@@ -261,16 +268,26 @@ impl From<&str> for RecipeId {
     }
 }
 
+/// For rstest magic conversions
 #[cfg(any(test, feature = "test"))]
-impl crate::test_util::Factory for RecipeId {
-    fn factory(_: ()) -> Self {
+impl FromStr for RecipeId {
+    type Err = ();
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        Ok::<_, ()>(s.to_owned().into())
+    }
+}
+
+#[cfg(any(test, feature = "test"))]
+impl slumber_util::Factory for RecipeId {
+    fn factory((): ()) -> Self {
         uuid::Uuid::new_v4().to_string().into()
     }
 }
 
 #[cfg(any(test, feature = "test"))]
-impl crate::test_util::Factory for Chain {
-    fn factory(_: ()) -> Self {
+impl slumber_util::Factory for Chain {
+    fn factory((): ()) -> Self {
         Self {
             id: "chain1".into(),
             source: ChainSource::Request {
@@ -577,8 +594,8 @@ impl Collection {
 }
 
 #[cfg(any(test, feature = "test"))]
-impl crate::test_util::Factory for Collection {
-    fn factory(_: ()) -> Self {
+impl slumber_util::Factory for Collection {
+    fn factory((): ()) -> Self {
         use crate::test_util::by_id;
         // Include a body in the recipe, so body-related behavior can be tested
         let recipe = Recipe {
@@ -600,9 +617,9 @@ impl crate::test_util::Factory for Collection {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::test_util::Factory;
     use indexmap::indexmap;
     use rstest::rstest;
+    use slumber_util::Factory;
 
     #[rstest]
     #[case::none(None, None, None)]
@@ -660,8 +677,8 @@ mod tests {
             headers.insert("content-type".into(), header.into());
         }
         let recipe = Recipe {
-            headers,
             body,
+            headers,
             ..Recipe::factory(())
         };
         let expected = expected.and_then(|value| value.parse::<Mime>().ok());
