@@ -1,12 +1,12 @@
 //! Recipe/folder tree structure
 
 use crate::collection::{
-    cereal::deserialize_id_map, Folder, HasId, Recipe, RecipeId,
+    Folder, HasId, Recipe, RecipeId, cereal::deserialize_id_map,
 };
 use anyhow::anyhow;
 use derive_more::From;
-use indexmap::{map::Values, IndexMap};
-use serde::{de::Error, Deserialize, Deserializer, Serialize};
+use indexmap::{IndexMap, map::Values};
+use serde::{Deserialize, Deserializer, Serialize, de::Error};
 use strum::EnumDiscriminants;
 use thiserror::Error;
 
@@ -66,7 +66,6 @@ impl IntoIterator for RecipeLookupKey {
 #[strum_discriminants(name(RecipeNodeType))]
 #[cfg_attr(any(test, feature = "test"), derive(PartialEq))]
 #[serde(rename_all = "snake_case", deny_unknown_fields)]
-#[allow(clippy::large_enum_variant)]
 pub enum RecipeNode {
     Folder(Folder),
     /// Rename this variant to match the `requests` field in the root and
@@ -277,17 +276,15 @@ impl RecipeNode {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::{
-        assert_err,
-        test_util::{by_id, Factory},
-    };
+    use crate::test_util::by_id;
     use indexmap::indexmap;
     use itertools::Itertools;
     use rstest::{fixture, rstest};
     use serde_yaml::{
-        value::{Tag, TaggedValue},
         Value,
+        value::{Tag, TaggedValue},
     };
+    use slumber_util::{Factory, assert_err};
 
     impl<const N: usize> From<[&str; N]> for RecipeLookupKey {
         fn from(value: [&str; N]) -> Self {
