@@ -5,12 +5,12 @@
 //! not a value, use [ContentType]. If you want to parse dynamically based on
 //! the response's metadata, use [ResponseRecord::parse_body].
 
-use crate::util::Mapping;
-use anyhow::{anyhow, Context};
+use anyhow::{Context, anyhow};
 use derive_more::{Deref, Display, From};
-use mime::{Mime, APPLICATION, JSON};
+use mime::{APPLICATION, JSON, Mime};
 use reqwest::header::{self, HeaderMap, HeaderValue};
 use serde::{Deserialize, Serialize};
+use slumber_util::Mapping;
 use std::{borrow::Cow, ffi::OsStr, fmt::Debug, path::Path};
 
 /// All supported content types. Each variant should have a corresponding
@@ -134,7 +134,7 @@ impl ContentType {
     pub fn value_to_string(self, value: &serde_json::Value) -> String {
         match self {
             ContentType::Json => match value {
-                serde_json::Value::Null => "".into(),
+                serde_json::Value::Null => String::new(),
                 serde_json::Value::String(s) => s.clone(),
                 other => other.to_string(),
             },
@@ -195,12 +195,13 @@ impl ResponseContent for Json {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::{assert_err, http::ResponseRecord, test_util::Factory};
+    use crate::http::ResponseRecord;
     use reqwest::header::{
-        HeaderMap, HeaderValue, InvalidHeaderValue, CONTENT_TYPE,
+        CONTENT_TYPE, HeaderMap, HeaderValue, InvalidHeaderValue,
     };
     use rstest::rstest;
     use serde_json::json;
+    use slumber_util::{Factory, assert_err};
     use std::ops::Deref;
 
     /// Test all content types and their variants
@@ -248,7 +249,7 @@ mod tests {
         assert_err!(
             ContentType::from_path(Path::new("turbo.ohno")),
             "Unknown extension `ohno`"
-        )
+        );
     }
 
     /// Test all content types
