@@ -1,19 +1,19 @@
 //! Import request collections from Insomnia. Based on the Insomnia v4 export
 //! format
 
-use anyhow::{anyhow, Context};
+use anyhow::{Context, anyhow};
 use indexmap::IndexMap;
 use itertools::Itertools;
 use mime::Mime;
 use reqwest::header;
-use serde::{de::Error as _, Deserialize, Deserializer};
+use serde::{Deserialize, Deserializer, de::Error as _};
 use slumber_core::{
     collection::{
         self, Chain, ChainId, ChainSource, Collection, Folder, HasId, Profile,
         ProfileId, Recipe, RecipeBody, RecipeId, RecipeNode, RecipeTree,
         SelectorMode,
     },
-    http::{content_type::ContentType, HttpMethod},
+    http::{HttpMethod, content_type::ContentType},
     template::{Identifier, Template},
     util::NEW_ISSUE_LINK,
 };
@@ -93,7 +93,6 @@ struct Grouped {
 
 #[derive(Debug, Deserialize)]
 #[serde(tag = "_type", rename_all = "snake_case")]
-#[allow(clippy::large_enum_variant)]
 enum Resource {
     /// Maps to a folder
     RequestGroup(RequestGroup),
@@ -322,7 +321,7 @@ impl From<Request> for RecipeNode {
                 error!(
                     "Error importing body for request `{id}`: {error}",
                     id = request.id
-                )
+                );
             })
             .ok()
             .flatten();
@@ -343,6 +342,7 @@ impl From<Request> for RecipeNode {
 
         RecipeNode::Recipe(Recipe {
             id: request.id.into(),
+            persist: true,
             name: Some(request.name),
             method: request.method,
             url: Template::raw(request.url),
@@ -530,21 +530,6 @@ fn build_recipe_tree(
     request_groups: Vec<RequestGroup>,
     requests: Vec<Request>,
 ) -> anyhow::Result<RecipeTree> {
-    // First, we want to match each parent with its children. Hashmap is fine
-    // because we won't be iterating over it
-    let mut children_map: HashMap<String, Vec<RecipeNode>> = request_groups
-        .into_iter()
-        .map(|request_group| {
-            (
-                request_group.parent_id.clone(),
-                RecipeNode::from(request_group),
-            )
-        })
-        .chain(requests.into_iter().map(|request| {
-            (request.parent_id.clone(), RecipeNode::from(request))
-        }))
-        .into_group_map();
-
     /// Recursively build the recipe tree by removing children from the given
     /// map, starting with a particular parent node
     fn build_tree(
@@ -570,6 +555,21 @@ fn build_recipe_tree(
         Ok(tree)
     }
 
+    // First, we want to match each parent with its children. Hashmap is fine
+    // because we won't be iterating over it
+    let mut children_map: HashMap<String, Vec<RecipeNode>> = request_groups
+        .into_iter()
+        .map(|request_group| {
+            (
+                request_group.parent_id.clone(),
+                RecipeNode::from(request_group),
+            )
+        })
+        .chain(requests.into_iter().map(|request| {
+            (request.parent_id.clone(), RecipeNode::from(request))
+        }))
+        .into_group_map();
+
     let tree = build_tree(&mut children_map, workspace_id)?;
     Ok(RecipeTree::new(tree)?)
 }
@@ -617,8 +617,8 @@ mod tests {
     use pretty_assertions::assert_eq;
     use rstest::rstest;
     use serde::de::DeserializeOwned;
-    use serde_test::{assert_de_tokens, assert_de_tokens_error, Token};
-    use slumber_core::test_util::test_data_dir;
+    use serde_test::{Token, assert_de_tokens, assert_de_tokens_error};
+    use slumber_util::test_data_dir;
     use std::{fmt::Debug, path::PathBuf};
 
     const INSOMNIA_FILE: &str = "insomnia.json";
@@ -733,6 +733,6 @@ mod tests {
                 Token::MapEnd,
             ],
             "data did not match any variant of untagged enum MapOption",
-        )
+        );
     }
 }
