@@ -11,7 +11,7 @@
 mod resolve;
 
 use crate::openapi::resolve::ReferenceResolver;
-use anyhow::{anyhow, Context};
+use anyhow::{Context, anyhow};
 use indexmap::IndexMap;
 use itertools::Itertools;
 use mime::Mime;
@@ -27,8 +27,9 @@ use slumber_core::{
     },
     http::HttpMethod,
     template::Template,
-    util::{ResultTraced, NEW_ISSUE_LINK},
+    util::NEW_ISSUE_LINK,
 };
+use slumber_util::ResultTraced;
 use std::{fs::File, iter, path::Path};
 use strum::IntoEnumIterator;
 use tracing::{debug, error, info, warn};
@@ -66,7 +67,7 @@ pub fn from_openapi(
             "Importer currently only supports OpenAPI v3.0, this spec is
                 version {openapi_version}. We'll try the import anyway, but you
                 may experience issues."
-        )
+        );
     }
 
     let profiles = build_profiles(servers);
@@ -198,7 +199,7 @@ fn build_recipe_tree(
                 // According to the spec, only *external* references can be used
                 // here, and we don't support those, so don't bother
                 // https://spec.openapis.org/oas/v3.0.3#path-item-object
-                error!("References not supported for path items `{reference}`")
+                error!("References not supported for path items `{reference}`");
             }
         }
     }
@@ -299,6 +300,7 @@ impl<'a> RecipeBuilder<'a> {
 
         Recipe {
             id: builder.id,
+            persist: true,
             name: Some(builder.name),
             method: builder.method,
             url,
@@ -365,7 +367,7 @@ impl<'a> RecipeBuilder<'a> {
                     error!(
                         "Unsupported type for path param `{}`",
                         parameter_data.name
-                    )
+                    );
                 }
                 Parameter::Cookie { parameter_data, .. } => {
                     error!(
@@ -387,7 +389,7 @@ impl<'a> RecipeBuilder<'a> {
             // don't care about the values for each scheme
             .unwrap_or_default()
             .into_iter()
-            .flat_map(|scheme| scheme.into_keys())
+            .flat_map(IndexMap::into_keys)
             // Resolve references, throwing away invalid ones
             .filter_map(|scheme_name| {
                 self.reference_resolver
@@ -639,7 +641,8 @@ mod tests {
     use pretty_assertions::assert_eq;
     use rstest::{fixture, rstest};
     use serde_json::json;
-    use slumber_core::{collection::Collection, test_util::test_data_dir};
+    use slumber_core::collection::Collection;
+    use slumber_util::test_data_dir;
     use std::{path::PathBuf, sync::OnceLock};
 
     const OPENAPIV3_FILE: &str = "openapiv3_petstore.yml";
