@@ -3,23 +3,31 @@
 #[cfg(test)]
 mod tests;
 
+use crate::{
+    context::TuiContext,
+    message::{Message, MessageSender},
+};
+use anyhow::Context;
+use async_trait::async_trait;
 use chrono::{DateTime, TimeDelta, Utc};
 use itertools::Itertools;
 use reqwest::StatusCode;
 use slumber_core::{
     collection::{ProfileId, RecipeId},
-    db::CollectionDatabase,
+    database::{CollectionDatabase, ProfileFilter},
     http::{
         Exchange, ExchangeSummary, RequestBuildError, RequestError, RequestId,
-        RequestRecord,
+        RequestRecord, RequestSeed,
     },
+    template::{HttpProvider, TemplateContext, TriggeredRequestError},
 };
 use std::{
-    collections::{hash_map::Entry, HashMap},
+    collections::{HashMap, hash_map::Entry},
     fmt::Debug,
     sync::Arc,
 };
-use tokio::task::JoinHandle;
+use strum::EnumDiscriminants;
+use tokio::{sync::oneshot, task::AbortHandle};
 use tracing::warn;
 
 /// Simple in-memory "database" for request state. This serves a few purposes:
@@ -66,14 +74,14 @@ impl RequestStore {
         id: RequestId,
         profile_id: Option<ProfileId>,
         recipe_id: RecipeId,
-        join_handle: JoinHandle<()>,
+        abort_handle: Option<AbortHandle>,
     ) {
         let state = RequestState::Building {
             id,
             start_time: Utc::now(),
             profile_id,
             recipe_id,
-            join_handle,
+            abort_handle,
         };
         self.requests.insert(id, state);
     }
@@ -83,12 +91,12 @@ impl RequestStore {
         self.replace(request.id, |state| {
             // Requests should go building->loading, but it's possible it got
             // cancelled right before this was called
-            if let RequestState::Building { join_handle, .. } = state {
+            if let RequestState::Building { abort_handle, .. } = state {
                 RequestState::Loading {
                     request,
                     // Reset timer
                     start_time: Utc::now(),
-                    join_handle,
+                    abort_handle,
                 }
             } else {
                 // Can't create loading state since we don't have a join handle
@@ -103,7 +111,10 @@ impl RequestStore {
 
     /// Mark a request as failed because of a build error. Return the updated
     /// state.
-    pub fn build_error(&mut self, error: RequestBuildError) -> &RequestState {
+    pub fn build_error(
+        &mut self,
+        error: Arc<RequestBuildError>,
+    ) -> &RequestState {
         // Use replace just to help catch bugs
         self.replace(error.id, |state| {
             // This indicates a bug or race condition (e.g. build cancelled as
@@ -120,7 +131,8 @@ impl RequestStore {
     }
 
     /// Mark a request as successful, i.e. we received a response. Return the
-    /// updated state.
+    /// updated state. Caller is responsible for persisting the exchange in the
+    /// DB.
     pub fn response(&mut self, exchange: Exchange) -> &RequestState {
         let response_state = RequestState::response(exchange);
         // Use replace just to help catch bugs
@@ -140,7 +152,7 @@ impl RequestStore {
 
     /// Mark a request as failed because of an HTTP error. Return the updated
     /// state.
-    pub fn request_error(&mut self, error: RequestError) -> &RequestState {
+    pub fn request_error(&mut self, error: Arc<RequestError>) -> &RequestState {
         // Use replace just to help catch bugs
         self.replace(error.request.id, |state| {
             // This indicates a bug or race condition (e.g. request cancelled as
@@ -166,9 +178,9 @@ impl RequestStore {
                 start_time,
                 profile_id,
                 recipe_id,
-                join_handle,
+                abort_handle: Some(abort_handle),
             } => {
-                join_handle.abort();
+                abort_handle.abort();
                 RequestState::Cancelled {
                     id,
                     recipe_id,
@@ -180,9 +192,9 @@ impl RequestStore {
             RequestState::Loading {
                 request,
                 start_time,
-                join_handle,
+                abort_handle: Some(abort_handle),
             } => {
-                join_handle.abort();
+                abort_handle.abort();
                 RequestState::Cancelled {
                     id,
                     recipe_id: request.recipe_id.clone(),
@@ -194,10 +206,7 @@ impl RequestStore {
             state => {
                 // If the request failed/finished while the cancel event was
                 // queued, don't do anything
-                warn!(
-                    request = ?state,
-                    "Cannot cancel request: not in building/loading state",
-                );
+                warn!(request = ?state, "Cannot cancel request");
                 state
             }
         })
@@ -228,18 +237,7 @@ impl RequestStore {
         profile_id: Option<&ProfileId>,
         recipe_id: &RecipeId,
     ) -> anyhow::Result<Option<&RequestState>> {
-        // Get the latest record in the DB
-        let exchange = self
-            .database
-            .get_latest_request(profile_id.into(), recipe_id)?;
-        if let Some(exchange) = exchange {
-            // Cache this record if it isn't already
-            self.requests
-                .entry(exchange.id)
-                // This is expensive because it parses the body, so avoid it if
-                // the record is already cached
-                .or_insert_with(|| RequestState::response(exchange));
-        }
+        self.cache_latest_exchange(profile_id, recipe_id)?;
 
         // Now that the know the most recent completed record is in our local
         // cache, find the most recent record of *any* kind
@@ -248,12 +246,58 @@ impl RequestStore {
             .requests
             .values()
             .filter(|state| {
-                state.profile_id() == profile_id
+                profile_id == state.profile_id()
                     && state.recipe_id() == recipe_id
             })
             .max_by_key(|state| state.request_metadata().start_time))
     }
 
+    /// Load the latest (by start time) _completed_ request for a specific
+    /// profile+recipe combo
+    pub fn load_latest_exchange(
+        &mut self,
+        profile_id: Option<&ProfileId>,
+        recipe_id: &RecipeId,
+    ) -> anyhow::Result<Option<&Exchange>> {
+        self.cache_latest_exchange(profile_id, recipe_id)?;
+
+        // Now that the know the most recent _persisted_ exchange is cached,
+        // find the most recent in the store. This will include unpersisted
+        // exchanges as well
+
+        Ok(self
+            .requests
+            .values()
+            .filter_map(|state| match state {
+                RequestState::Response { exchange }
+                    if profile_id == state.profile_id()
+                        && state.recipe_id() == recipe_id =>
+                {
+                    Some(exchange)
+                }
+                _ => None,
+            })
+            .max_by_key(|exchange| exchange.start_time))
+    }
+
+    /// Load the most recent matching exchange from the DB and cache it here
+    fn cache_latest_exchange(
+        &mut self,
+        profile_id: Option<&ProfileId>,
+        recipe_id: &RecipeId,
+    ) -> anyhow::Result<()> {
+        let exchange = self
+            .database
+            .get_latest_request(profile_id.into(), recipe_id)?;
+        if let Some(exchange) = exchange {
+            // Cache this record if it isn't already
+            self.requests
+                .entry(exchange.id)
+                .or_insert(RequestState::response(exchange));
+        }
+        Ok(())
+    }
+
     /// Load all historical requests for a recipe+profile, then return the
     /// *entire* set of requests, including in-progress ones. Returned requests
     /// are just summaries, not the full request. This is intended for list
@@ -268,7 +312,7 @@ impl RequestStore {
         // store, because they don't include request/response data
         let loaded = self
             .database
-            .get_all_requests(profile_id.into(), recipe_id)?;
+            .get_recipe_requests(profile_id.into(), recipe_id)?;
 
         // Find what we have in memory already
         let iter = self
@@ -289,14 +333,47 @@ impl RequestStore {
         Ok(iter)
     }
 
-    /// Is the given request either building or loading?
-    pub fn is_in_progress(&self, id: RequestId) -> bool {
+    /// Is the given request either building or loading, and does it have an
+    /// abort handle? Triggered requests (nested within another request's
+    /// render) cannot be cancelled independently.
+    pub fn can_cancel(&self, id: RequestId) -> bool {
         matches!(
             self.get(id),
-            Some(RequestState::Building { .. } | RequestState::Loading { .. },)
+            Some(
+                RequestState::Building {
+                    abort_handle: Some(_),
+                    ..
+                } | RequestState::Loading {
+                    abort_handle: Some(_),
+                    ..
+                }
+            )
         )
     }
 
+    /// Delete all requests for a specific recipe+profile combo. Return the
+    /// number of deleted requests
+    pub fn delete_recipe_requests(
+        &mut self,
+        profile_filter: ProfileFilter,
+        recipe_id: &RecipeId,
+    ) -> anyhow::Result<usize> {
+        self.requests.retain(|_, state| {
+            // Keep items that _don't_ match
+            !(state.recipe_id() == recipe_id
+                && profile_filter.matches(state.profile_id()))
+        });
+        self.database
+            .delete_recipe_requests(profile_filter, recipe_id)
+    }
+
+    /// Delete a single request from the store _and_ the database
+    pub fn delete_request(&mut self, id: RequestId) -> anyhow::Result<()> {
+        self.requests.remove(&id);
+        self.database.delete_request(id)?;
+        Ok(())
+    }
+
     /// Replace a request state in the store with new state, by mapping it
     /// through a function. This assumes the request state is supposed to be in
     /// the state, so it logs a message if it isn't (panics in debug mode). This
@@ -320,10 +397,101 @@ impl RequestStore {
     }
 }
 
+/// An [HttpProvider] that uses the request store (and by extension the
+/// database) to access and persist HTTP requests. This defers operations on the
+/// request store through the message pipeline, because we can't have direct
+/// access to the request store from a template rendering task. We could solve
+/// this with `Rc<RefCell>` instead, but that ends up polluting the request
+/// store type signatures a lot. This is self-contained with minimal perf impact
+#[derive(Debug)]
+pub struct TuiHttpProvider {
+    messages_tx: MessageSender,
+    /// Are we rendering request previews, or the real deal? This controls
+    /// whether we'll send triggered requests or not
+    preview: bool,
+}
+
+impl TuiHttpProvider {
+    pub fn new(messages_tx: MessageSender, preview: bool) -> Self {
+        Self {
+            messages_tx,
+            preview,
+        }
+    }
+}
+
+#[async_trait]
+impl HttpProvider for TuiHttpProvider {
+    async fn get_latest_request(
+        &self,
+        profile_id: Option<&ProfileId>,
+        recipe_id: &RecipeId,
+    ) -> anyhow::Result<Option<Exchange>> {
+        // Defer the fetch into a message because we can't access the request
+        // store from another task
+        let (tx, rx) = oneshot::channel();
+        self.messages_tx.send(Message::HttpGetLatest {
+            profile_id: profile_id.cloned(),
+            recipe_id: recipe_id.clone(),
+            channel: tx.into(),
+        });
+        rx.await.context("Error fetching request")
+    }
+
+    async fn send_request(
+        &self,
+        seed: RequestSeed,
+        template_context: &TemplateContext,
+    ) -> Result<Exchange, TriggeredRequestError> {
+        if self.preview {
+            // Previews shouldn't have side effects
+            Err(TriggeredRequestError::NotAllowed)
+        } else {
+            // We'll report start updates back to the main loop as we go, so the
+            // chained request is visible in the UI. This isn't strictly
+            // necessary, but it's easy and keeps the UI in sync with the
+            // underlying state
+            let request_id = seed.id;
+            let profile_id = template_context.selected_profile.clone();
+            let recipe_id = seed.recipe_id.clone();
+
+            self.messages_tx.send(Message::HttpBuildingTriggered {
+                id: request_id,
+                profile_id,
+                recipe_id,
+            });
+
+            let ticket = TuiContext::get()
+                .http_engine
+                .build(seed, template_context)
+                .await
+                .map_err(Arc::new)
+                .inspect_err(|error| {
+                    // Report error to the TUI
+                    self.messages_tx.send(Message::HttpBuildError {
+                        error: Arc::clone(error),
+                    });
+                })?;
+
+            // Build successful, send it out
+            self.messages_tx.send(Message::HttpLoading {
+                request: Arc::clone(ticket.record()),
+            });
+
+            // Clone the exchange so we can persist it in the DB/store and
+            // still return it
+            let result = ticket.send().await.map_err(Arc::new);
+            self.messages_tx.send(Message::HttpComplete(result.clone()));
+            result.map_err(TriggeredRequestError::Send)
+        }
+    }
+}
+
 /// State of an HTTP response, which can be in various states of
 /// completion/failure. Each request *recipe* should have one request state
 /// stored in the view at a time.
-#[derive(Debug)]
+#[derive(Debug, EnumDiscriminants)]
+#[strum_discriminants(name(RequestStateType))]
 pub enum RequestState {
     /// The request is being built. Typically this is very fast, but can be
     /// slow if a chain source takes a while.
@@ -332,11 +500,14 @@ pub enum RequestState {
         start_time: DateTime<Utc>,
         profile_id: Option<ProfileId>,
         recipe_id: RecipeId,
-        join_handle: JoinHandle<()>,
+        /// A handle to abort the task running the request. Used to cancel the
+        /// request. `None` for triggered requests, because they don't run at
+        /// the root of a task and therefore can't be aborted independently.
+        abort_handle: Option<AbortHandle>,
     },
 
     /// Something went wrong during the build :(
-    BuildError { error: RequestBuildError },
+    BuildError { error: Arc<RequestBuildError> },
 
     /// Request is in flight, or is *about* to be sent. There's no way to
     /// initiate a request that doesn't immediately launch it, so Loading is
@@ -346,7 +517,10 @@ pub enum RequestState {
         /// pointer to the request as well
         request: Arc<RequestRecord>,
         start_time: DateTime<Utc>,
-        join_handle: JoinHandle<()>,
+        /// A handle to abort the task running the request. Used to cancel the
+        /// request. `None` for triggered requests, because they don't run at
+        /// the root of a task and therefore can't be aborted independently.
+        abort_handle: Option<AbortHandle>,
     },
 
     /// User cancelled the request mid-flight. We don't store the request here,
@@ -367,7 +541,11 @@ pub enum RequestState {
     Response { exchange: Exchange },
 
     /// Error occurred sending the request or receiving the response.
-    RequestError { error: RequestError },
+    RequestError {
+        /// This needs an `Arc` so it can be shared with the template engine in
+        /// the case of triggered chained requests
+        error: Arc<RequestError>,
+    },
 }
 
 impl RequestState {
@@ -422,23 +600,11 @@ impl RequestState {
             },
 
             // Error states
-            Self::BuildError {
-                error:
-                    RequestBuildError {
-                        start_time,
-                        end_time,
-                        ..
-                    },
-            }
-            | Self::RequestError {
-                error:
-                    RequestError {
-                        start_time,
-                        end_time,
-                        ..
-                    },
-            }
-            | Self::Cancelled {
+            Self::BuildError { error } => RequestMetadata {
+                start_time: error.start_time,
+                end_time: Some(error.end_time),
+            },
+            Self::Cancelled {
                 start_time,
                 end_time,
                 ..
@@ -446,6 +612,10 @@ impl RequestState {
                 start_time: *start_time,
                 end_time: Some(*end_time),
             },
+            Self::RequestError { error } => RequestMetadata {
+                start_time: error.start_time,
+                end_time: Some(error.end_time),
+            },
 
             // Completed
             Self::Response { exchange, .. } => RequestMetadata {
@@ -484,14 +654,14 @@ impl PartialEq for RequestState {
                     start_time: l_start_time,
                     profile_id: l_profile_id,
                     recipe_id: l_recipe_id,
-                    join_handle: _,
+                    abort_handle: _,
                 },
                 Self::Building {
                     id: r_id,
                     start_time: r_start_time,
                     profile_id: r_profile_id,
                     recipe_id: r_recipe_id,
-                    join_handle: _,
+                    abort_handle: _,
                 },
             ) => {
                 l_id == r_id
@@ -507,12 +677,12 @@ impl PartialEq for RequestState {
                 Self::Loading {
                     request: l_request,
                     start_time: l_start_time,
-                    join_handle: _,
+                    abort_handle: _,
                 },
                 Self::Loading {
                     request: r_request,
                     start_time: r_start_time,
-                    join_handle: _,
+                    abort_handle: _,
                 },
             ) => l_request == r_request && l_start_time == r_start_time,
             (
@@ -586,7 +756,7 @@ pub struct ResponseMetadata {
 /// A simplified version of [RequestState], which only stores metadata. This is
 /// useful when you want to show a list of requests and don't need the entire
 /// request/response data for each one.
-#[derive(Debug)]
+#[derive(Debug, PartialEq)]
 pub enum RequestStateSummary {
     Building {
         id: RequestId,
@@ -703,7 +873,7 @@ impl From<&RequestState> for RequestStateSummary {
                 end_time: *end_time,
             },
             RequestState::Response { exchange } => {
-                Self::Response(exchange.into())
+                Self::Response(exchange.summary())
             }
             RequestState::RequestError { error } => Self::RequestError {
                 id: error.request.id,
