@@ -3,20 +3,22 @@ use crate::{
     message::{Message, MessageSender},
     view::{Confirm, ViewContext},
 };
-use anyhow::{bail, Context};
+use anyhow::{Context, bail};
 use bytes::Bytes;
-use crossterm::event;
-use editor_command::EditorBuilder;
-use futures::{future, FutureExt};
-use mime::Mime;
-use slumber_core::{
-    template::Prompt,
-    util::{doc_link, paths::expand_home, ResultTraced},
+use crossterm::{
+    event::{self, DisableMouseCapture, EnableMouseCapture},
+    terminal::{EnterAlternateScreen, LeaveAlternateScreen},
 };
+use editor_command::EditorBuilder;
+use futures::{FutureExt, future};
+use mime::Mime;
+use slumber_core::{template::Prompt, util::doc_link};
+use slumber_util::{ResultTraced, paths::expand_home};
 use std::{
     env,
+    fs::{self, File},
     future::Future,
-    io,
+    io::{self, Write},
     ops::Deref,
     path::{Path, PathBuf},
     process::{Command, Stdio},
@@ -31,7 +33,7 @@ use tokio::{
     task::{self, JoinHandle},
 };
 use tokio_util::sync::CancellationToken;
-use tracing::{debug, debug_span, error, info, warn};
+use tracing::{debug, debug_span, error, info, info_span, warn};
 use uuid::Uuid;
 
 /// Token to manage cancellation of background tasks
@@ -75,6 +77,137 @@ impl Flag {
     }
 }
 
+/// A temporary file. The file is created with a random name when the struct is
+/// initialized, and deleted when the struct is dropped.
+#[derive(Debug)]
+pub struct TempFile {
+    path: PathBuf,
+}
+
+impl TempFile {
+    /// Create a new temporary file with the given contents
+    pub fn new(contents: &[u8]) -> anyhow::Result<Self> {
+        Self::with_file(|file| file.write_all(contents))
+    }
+
+    /// Create a new temporary file and call a function to initialize it with
+    /// data. This is used for writing a ratatui `Text` object to a file, which
+    /// isn't accessible as a single chunk of bytes.
+    pub fn with_file(
+        mut writer: impl FnMut(&mut File) -> io::Result<()>,
+    ) -> anyhow::Result<Self> {
+        let path = env::temp_dir().join(format!("slumber-{}", Uuid::new_v4()));
+        let mut file = std::fs::OpenOptions::new()
+            .create(true)
+            .write(true)
+            .truncate(true)
+            .open(&path)
+            .with_context(|| {
+                format!("Error creating temporary file {path:?}")
+            })?;
+        writer(&mut file).with_context(|| {
+            format!("Error writing to temporary file {path:?}")
+        })?;
+        Ok(Self { path })
+    }
+
+    pub fn path(&self) -> &Path {
+        &self.path
+    }
+}
+
+impl Drop for TempFile {
+    fn drop(&mut self) {
+        let _ = fs::remove_file(&self.path)
+            .with_context(|| {
+                format!("Error deleting temporary file {:?}", self.path)
+            })
+            .traced();
+    }
+}
+
+/// Run a **blocking** subprocess that will take over the terminal. Used
+/// for opening an external editor or pager. Useful for terminal editors since
+/// they'll take over the whole screen. Potentially annoying for GUI editors
+/// that block, but we'll just hope the command is fire-and-forget. If this
+/// becomes an issue we can try to detect if the subprocess took over the
+/// terminal and cut it loose if not, or add a config field for it.
+pub fn yield_terminal(
+    mut command: Command,
+    messages_tx: &MessageSender,
+) -> anyhow::Result<()> {
+    let span = info_span!("Running command", ?command).entered();
+    let error_context = format!("Error spawning command `{command:?}`");
+
+    // Clear the terminal so the buffer is empty. This forces a total redraw
+    // when we take it back over. Otherwise ratatui would think the screen is
+    // still intact and not draw anything
+    messages_tx.send(Message::ClearTerminal);
+
+    // Reset terminal to normal
+    restore_terminal()?;
+
+    // Run the command. Make sure to perform cleanup even if the command
+    // failed
+    let command_result = command
+        .status()
+        .map_err(anyhow::Error::from)
+        .and_then(|status| {
+            if status.success() {
+                info!(status = status.code(), "Command succeeded");
+                Ok(())
+            } else {
+                // It would be nice to log stdout/stderr here, but we can't
+                // capture them because some commands (e.g. `less`) will behave
+                // differently when redirected
+                error!(status = status.code(), "Command failed");
+                // Show the error to the user
+                Err(anyhow::anyhow!("Command failed with status {}", status))
+            }
+        })
+        .context(error_context);
+
+    // Some editors *cough* vim *cough* dump garbage to the event buffer on
+    // exit. I've never figured out what actually causes it, but a simple
+    // solution is to dump all events in the buffer before returning to
+    // Slumber. It's possible we lose some real user input here (e.g. if
+    // other events were queued behind the event to open the editor).
+    clear_event_buffer();
+    initialize_terminal()?; // Take it back over
+    drop(span);
+
+    // Redraw immediately. The main loop will probably be in the tick
+    // timeout when we go back to it, so that adds a 250ms delay to
+    // redrawing the screen that we want to skip.
+    messages_tx.send(Message::Draw);
+
+    command_result
+}
+
+/// Set up terminal for TUI
+pub fn initialize_terminal() -> anyhow::Result<()> {
+    debug!("Initializing terminal");
+    crossterm::terminal::enable_raw_mode()?;
+    crossterm::execute!(
+        io::stdout(),
+        EnterAlternateScreen,
+        EnableMouseCapture
+    )?;
+    Ok(())
+}
+
+/// Return terminal to initial state
+pub fn restore_terminal() -> anyhow::Result<()> {
+    debug!("Restoring terminal");
+    crossterm::terminal::disable_raw_mode()?;
+    crossterm::execute!(
+        io::stdout(),
+        LeaveAlternateScreen,
+        DisableMouseCapture
+    )?;
+    Ok(())
+}
+
 /// Clear all input events in the terminal event buffer
 pub fn clear_event_buffer() {
     while let Ok(true) = event::poll(Duration::from_millis(0)) {
@@ -87,7 +220,7 @@ pub fn clear_event_buffer() {
 #[cfg(unix)]
 pub async fn signals() -> anyhow::Result<()> {
     use itertools::Itertools;
-    use tokio::signal::unix::{signal, Signal, SignalKind};
+    use tokio::signal::unix::{Signal, SignalKind, signal};
 
     let signals: Vec<(Signal, SignalKind)> = [
         SignalKind::interrupt(),
@@ -127,19 +260,6 @@ pub async fn signals() -> anyhow::Result<()> {
     Ok(())
 }
 
-/// Get a path to a random temp file
-pub fn temp_file() -> PathBuf {
-    env::temp_dir().join(format!("slumber-{}", Uuid::new_v4()))
-}
-
-/// Delete a file. If it fails, trace and move on because it's not important
-/// enough to bother the user
-pub fn delete_temp_file(path: &Path) {
-    let _ = std::fs::remove_file(path)
-        .with_context(|| format!("Error deleting file {path:?}"))
-        .traced();
-}
-
 /// Spawn a task on the main thread. Most tasks can use this because the app is
 /// generally I/O bound, so we can handle all async stuff on a single thread.
 /// The UI will be redrawn when the task is done. This redraw may be redundant,
@@ -147,12 +267,12 @@ pub fn delete_temp_file(path: &Path) {
 pub fn spawn(future: impl 'static + Future<Output = ()>) -> JoinHandle<()> {
     task::spawn_local(async move {
         select! {
-            _ = future => {
+            () = future => {
                 // Assume the task updated _something_ visible to the user,
                 // so trigger a redraw here
-                ViewContext::messages_tx().send(Message::Tick);
+                ViewContext::messages_tx().send(Message::Draw);
             },
-            _ = CANCEL_TOKEN.cancelled() => {},
+            () = CANCEL_TOKEN.cancelled() => {},
         }
     })
 }
@@ -370,7 +490,10 @@ async fn prompt(
 }
 
 /// Ask the user a yes/no question and wait for a response
-async fn confirm(messages_tx: &MessageSender, message: impl ToString) -> bool {
+pub async fn confirm(
+    messages_tx: &MessageSender,
+    message: impl ToString,
+) -> bool {
     let (tx, rx) = oneshot::channel();
     let confirm = Confirm {
         message: message.to_string(),
@@ -384,13 +507,10 @@ async fn confirm(messages_tx: &MessageSender, message: impl ToString) -> bool {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::test_util::{harness, TestHarness};
+    use crate::test_util::{TestHarness, harness};
     use rstest::rstest;
     use slumber_config::CommandsConfig;
-    use slumber_core::{
-        assert_matches,
-        test_util::{temp_dir, TempDir},
-    };
+    use slumber_util::{TempDir, assert_matches, temp_dir};
     use tokio::fs;
 
     /// Test various cases of save_file
