@@ -14,9 +14,9 @@ use crate::{
 };
 use itertools::Itertools;
 use ratatui::{
+    Frame,
     layout::Constraint,
     text::{Line, Span},
-    Frame,
 };
 use slumber_config::Action;
 use std::fmt::Display;
@@ -69,11 +69,10 @@ impl Modal for ActionsModal {
 
     fn on_close(self: Box<Self>, submitted: bool) {
         if submitted {
-            let action = self
-                .actions
-                .into_data()
-                .into_selected()
-                .expect("User submitted something");
+            let Some(action) = self.actions.into_data().into_selected() else {
+                // Possible if the action list is empty
+                return;
+            };
             // Emit an event on behalf of the component that supplied this
             // action. The component will use its own supplied emitter ID to
             // consume the event
@@ -117,7 +116,7 @@ impl EventHandler for ActionsModal {
 }
 
 impl Draw for ActionsModal {
-    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
+    fn draw(&self, frame: &mut Frame, (): (), metadata: DrawMetadata) {
         self.actions.draw(
             frame,
             List::from(self.actions.data()),
@@ -165,7 +164,8 @@ impl MenuAction {
 }
 
 impl Generate for &MenuAction {
-    type Output<'this> = Span<'this>
+    type Output<'this>
+        = Span<'this>
     where
         Self: 'this;
 
@@ -204,7 +204,7 @@ pub trait IntoMenuAction<Data>: Display + LocalEvent {
 mod tests {
     use super::*;
     use crate::{
-        test_util::{harness, terminal, TestHarness, TestTerminal},
+        test_util::{TestHarness, TestTerminal, harness, terminal},
         view::test_util::TestComponent,
     };
     use crossterm::event::KeyCode;
@@ -226,7 +226,7 @@ mod tests {
     }
 
     impl Draw for Actionable {
-        fn draw(&self, _: &mut Frame, _: (), _: DrawMetadata) {}
+        fn draw(&self, _: &mut Frame, (): (), _: DrawMetadata) {}
     }
 
     impl ToEmitter<TestMenuAction> for Actionable {
@@ -244,11 +244,11 @@ mod tests {
     }
 
     impl IntoMenuAction<()> for TestMenuAction {
-        fn enabled(&self, _: &()) -> bool {
+        fn enabled(&self, &(): &()) -> bool {
             !matches!(self, Self::Disablify)
         }
 
-        fn shortcut(&self, _: &()) -> Option<Action> {
+        fn shortcut(&self, &(): &()) -> Option<Action> {
             match self {
                 Self::Shortcutticated => Some(Action::Edit),
                 _ => None,
