@@ -10,9 +10,9 @@ use crate::{
     },
 };
 use ratatui::{
+    Frame,
     layout::{Constraint, Flex, Layout},
     text::Span,
-    Frame,
 };
 use slumber_config::Action;
 
@@ -24,8 +24,9 @@ pub struct Button<'a> {
     has_focus: bool,
 }
 
-impl<'a> Generate for Button<'a> {
-    type Output<'this> = Span<'this>
+impl Generate for Button<'_> {
+    type Output<'this>
+        = Span<'this>
     where
         Self: 'this;
 
@@ -74,19 +75,13 @@ impl<T: FixedSelect> EventHandler for ButtonGroup<T> {
 }
 
 impl<T: FixedSelect> Draw for ButtonGroup<T> {
-    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
-        // The button width is based on the longest button
-        let width = self
-            .select
-            .items()
-            .map(|button| button.to_string().len())
-            .max()
-            .unwrap_or(0) as u16;
-        let (areas, _) = Layout::horizontal(
-            self.select.items().map(|_| Constraint::Length(width)),
-        )
-        .flex(Flex::SpaceAround)
-        .split_with_spacers(metadata.area());
+    fn draw(&self, frame: &mut Frame, (): (), metadata: DrawMetadata) {
+        let (areas, _) =
+            Layout::horizontal(self.select.items().map(|button| {
+                Constraint::Length(button.to_string().len() as u16)
+            }))
+            .flex(Flex::SpaceBetween)
+            .split_with_spacers(metadata.area());
 
         for (button, area) in self.select.items().zip(areas.iter()) {
             frame.render_widget(
@@ -96,7 +91,7 @@ impl<T: FixedSelect> Draw for ButtonGroup<T> {
                 }
                 .generate(),
                 *area,
-            )
+            );
         }
     }
 }
