@@ -1,6 +1,7 @@
 use crate::{
     context::TuiContext,
     view::{
+        Component, ViewContext,
         context::UpdateContext,
         draw::{Draw, DrawMetadata},
         event::{
@@ -8,15 +9,14 @@ use crate::{
             ToEmitter,
         },
         util::centered_rect,
-        Component, ViewContext,
     },
 };
 use ratatui::{
+    Frame,
     layout::Margin,
     prelude::Constraint,
     text::Line,
     widgets::{Block, Borders, Clear},
-    Frame,
 };
 use slumber_config::Action;
 use std::{collections::VecDeque, fmt::Debug, ops::DerefMut};
@@ -161,7 +161,7 @@ impl EventHandler for ModalQueue {
                 // eat the Quit action here because it's  (hopefully)
                 // intuitive and consistent with other TUIs
                 Action::Cancel | Action::Quit if self.is_open() => {
-                    self.close(false)
+                    self.close(false);
                 }
                 Action::Submit if self.is_open() => self.close(true),
                 _ => propagate.set(),
@@ -199,7 +199,7 @@ impl EventHandler for ModalQueue {
 }
 
 impl Draw for ModalQueue {
-    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
+    fn draw(&self, frame: &mut Frame, (): (), metadata: DrawMetadata) {
         if let Some(modal) = self.queue.front() {
             let styles = &TuiContext::get().styles;
             let (width, height) = modal.data().dimensions();
@@ -212,8 +212,8 @@ impl Draw for ModalQueue {
             let mut area = centered_rect(width, height, metadata.area());
             let x_buffer = margin.horizontal + 1;
             let y_buffer = margin.vertical + 1;
-            area.x -= x_buffer;
-            area.y -= y_buffer;
+            area.x = area.x.saturating_sub(x_buffer);
+            area.y = area.y.saturating_sub(y_buffer);
             area.width += x_buffer * 2;
             area.height += y_buffer * 2;
 
