@@ -12,10 +12,10 @@ use crate::{
 use crossterm::event::{KeyCode, KeyModifiers};
 use persisted::PersistedContainer;
 use ratatui::{
+    Frame,
     layout::Rect,
     text::{Line, Masked, Text},
     widgets::{Paragraph, ScrollbarOrientation},
-    Frame,
 };
 use slumber_config::Action;
 use std::{cell::Cell, mem};
@@ -157,7 +157,7 @@ impl TextBox {
             }
             _ => return false, // Event should be propagated
         };
-        // If text _content_ changed, trigger the auto-submit debounce
+        // If text _content_ changed, trigger the change event
         if text_changed {
             self.change();
         }
@@ -363,23 +363,23 @@ impl TextState {
     /// Delete character immediately left of the cursor. Return `true` if text
     /// was modified
     fn delete_left(&mut self) -> bool {
-        if !self.is_at_home() {
+        if self.is_at_home() {
+            false
+        } else {
             self.left();
             self.text.remove(self.cursor);
             true
-        } else {
-            false
         }
     }
 
     /// Delete character immediately rightof the cursor. Return `true` if text
     /// was modified
     fn delete_right(&mut self) -> bool {
-        if !self.is_at_end() {
+        if self.is_at_end() {
+            false
+        } else {
             self.text.remove(self.cursor);
             true
-        } else {
-            false
         }
     }
 
@@ -465,12 +465,12 @@ struct TextStats {
 mod tests {
     use super::*;
     use crate::{
-        test_util::{harness, terminal, TestHarness, TestTerminal},
+        test_util::{TestHarness, TestTerminal, harness, terminal},
         view::test_util::TestComponent,
     };
     use ratatui::{layout::Margin, text::Span};
     use rstest::rstest;
-    use slumber_core::assert_matches;
+    use slumber_util::assert_matches;
 
     /// Create a span styled as the cursor
     fn cursor(text: &str) -> Span {
