@@ -9,12 +9,13 @@ use crate::{
 };
 use itertools::Itertools;
 use ratatui::{
+    Frame,
     layout::{Alignment, Constraint, Layout},
     text::{Line, Text},
-    Frame,
 };
 use slumber_config::{Action, Config, InputBinding};
-use slumber_core::util::{doc_link, paths};
+use slumber_core::util::doc_link;
+use slumber_util::paths;
 
 const CRATE_VERSION: &str = env!("CARGO_PKG_VERSION");
 
@@ -23,7 +24,8 @@ const CRATE_VERSION: &str = env!("CARGO_PKG_VERSION");
 pub struct HelpFooter;
 
 impl Generate for HelpFooter {
-    type Output<'this> = Text<'this>
+    type Output<'this>
+        = Text<'this>
     where
         Self: 'this;
 
@@ -83,7 +85,7 @@ impl Modal for HelpModal {
 impl EventHandler for HelpModal {}
 
 impl Draw for HelpModal {
-    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
+    fn draw(&self, frame: &mut Frame, (): (), metadata: DrawMetadata) {
         // Create layout
         let [collection_area, _, keybindings_area] = Layout::vertical([
             Constraint::Length(Self::GENERAL_LENGTH + 1),
