@@ -1,31 +1,39 @@
 //! Miscellaneous components. They have specific purposes and therefore aren't
 //! generic/utility, but don't fall into a clear category.
 
-use crate::view::{
-    common::{
-        button::ButtonGroup,
-        list::List,
-        modal::{IntoModal, Modal},
-        text_box::{TextBox, TextBoxEvent, TextBoxProps},
+use crate::{
+    util::ResultReported,
+    view::{
+        Confirm, ModalPriority, ViewContext,
+        common::{
+            button::ButtonGroup,
+            list::List,
+            modal::{IntoModal, Modal},
+            text_box::{TextBox, TextBoxEvent, TextBoxProps},
+        },
+        component::Component,
+        context::UpdateContext,
+        draw::{Draw, DrawMetadata, Generate},
+        event::{Child, Event, EventHandler, OptionEvent, ToEmitter},
+        state::{
+            Notification,
+            select::{SelectState, SelectStateEvent, SelectStateEventType},
+        },
     },
-    component::Component,
-    context::UpdateContext,
-    draw::{Draw, DrawMetadata, Generate},
-    event::{Child, Event, EventHandler, OptionEvent, ToEmitter},
-    state::{
-        select::{SelectState, SelectStateEvent, SelectStateEventType},
-        Notification,
-    },
-    Confirm, ModalPriority,
 };
 use derive_more::Display;
 use ratatui::{
+    Frame,
     prelude::Constraint,
     text::{Line, Text},
     widgets::Paragraph,
-    Frame,
 };
-use slumber_core::template::{Prompt, Select};
+use slumber_core::{
+    collection::{ProfileId, RecipeId},
+    database::ProfileFilter,
+    http::RequestId,
+    template::{Prompt, Select},
+};
 use std::fmt::Debug;
 use strum::{EnumCount, EnumIter};
 
@@ -51,7 +59,7 @@ impl Modal for ErrorModal {
 impl EventHandler for ErrorModal {}
 
 impl Draw for ErrorModal {
-    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
+    fn draw(&self, frame: &mut Frame, (): (), metadata: DrawMetadata) {
         frame.render_widget(self.0.generate(), metadata.area());
     }
 }
@@ -135,7 +143,7 @@ impl EventHandler for TextBoxModal {
 }
 
 impl Draw for TextBoxModal {
-    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
+    fn draw(&self, frame: &mut Frame, (): (), metadata: DrawMetadata) {
         self.text_box.draw(
             frame,
             TextBoxProps::default(),
@@ -199,8 +207,11 @@ impl Modal for SelectListModal {
         // Do some simple math to size the select modal correctly to our
         // underlying data
         let options = self.options.data();
-        let longest_option =
-            options.items().map(|s| s.len()).max().unwrap_or(10);
+        let longest_option = options
+            .items()
+            .map(std::string::String::len)
+            .max()
+            .unwrap_or(10);
         // find our widest string to appropriately set width
         let width = std::cmp::max(self.title.len(), longest_option);
         (
@@ -213,13 +224,11 @@ impl Modal for SelectListModal {
         // The modal is closed, but only submit the value if it was closed
         // because the user selected a value (submitted).
         if submitted {
-            // Return the user's value and close the prompt
-            (self.on_submit)(
-                self.options
-                    .into_data()
-                    .into_selected()
-                    .expect("User submitted something"),
-            );
+            // Return the user's value and close the prompt. Value can be empty
+            // if the select list is empty
+            let selected =
+                self.options.into_data().into_selected().unwrap_or_default();
+            (self.on_submit)(selected);
         }
     }
 }
@@ -239,7 +248,7 @@ impl EventHandler for SelectListModal {
 }
 
 impl Draw for SelectListModal {
-    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
+    fn draw(&self, frame: &mut Frame, (): (), metadata: DrawMetadata) {
         // Empty state
         let options = self.options.data();
         if options.is_empty() {
@@ -261,11 +270,21 @@ impl IntoModal for Select {
 
     fn into_modal(self) -> Self::Target {
         SelectListModal::new(self.message, self.options, |response| {
-            self.channel.respond(response)
+            self.channel.respond(response);
         })
     }
 }
 
+/// Buttons in a yes/no confirmation modal
+#[derive(
+    Copy, Clone, Debug, Default, Display, EnumCount, EnumIter, PartialEq,
+)]
+pub enum ConfirmButton {
+    No,
+    #[default]
+    Yes,
+}
+
 /// Inner state for the prompt modal
 #[derive(derive_more::Debug)]
 pub struct ConfirmModal {
@@ -280,17 +299,9 @@ pub struct ConfirmModal {
     on_submit: Box<dyn 'static + FnOnce(bool)>,
 }
 
-/// Buttons in the confirmation modal
-#[derive(
-    Copy, Clone, Debug, Default, Display, EnumCount, EnumIter, PartialEq,
-)]
-enum ConfirmButton {
-    No,
-    #[default]
-    Yes,
-}
-
 impl ConfirmModal {
+    const MIN_WIDTH: u16 = 24;
+
     pub fn new(title: String, on_submit: impl 'static + FnOnce(bool)) -> Self {
         Self {
             title,
@@ -309,7 +320,9 @@ impl Modal for ConfirmModal {
     fn dimensions(&self) -> (Constraint, Constraint) {
         (
             // Add some arbitrary padding
-            Constraint::Length((self.title.len() + 4) as u16),
+            Constraint::Length(
+                Self::MIN_WIDTH.max((self.title.len() + 4) as u16),
+            ),
             Constraint::Length(1),
         )
     }
@@ -326,6 +339,8 @@ impl EventHandler for ConfirmModal {
         event.opt().emitted(self.buttons.to_emitter(), |button| {
             // When user selects a button, send the response and close
             self.answer = button == ConfirmButton::Yes;
+            // If the user answers, then they submitted a response, even if the
+            // answer was no
             self.close(true);
         })
     }
@@ -336,7 +351,7 @@ impl EventHandler for ConfirmModal {
 }
 
 impl Draw for ConfirmModal {
-    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
+    fn draw(&self, frame: &mut Frame, (): (), metadata: DrawMetadata) {
         self.buttons.draw(frame, (), metadata.area(), true);
     }
 }
@@ -346,11 +361,158 @@ impl IntoModal for Confirm {
 
     fn into_modal(self) -> Self::Target {
         ConfirmModal::new(self.message, |response| {
-            self.channel.respond(response)
+            self.channel.respond(response);
         })
     }
 }
 
+/// Confirmation modal to delete a single request
+#[derive(Debug)]
+pub struct DeleteRequestModal {
+    request_id: RequestId,
+    buttons: Component<ButtonGroup<ConfirmButton>>,
+}
+
+impl DeleteRequestModal {
+    pub fn new(request_id: RequestId) -> Self {
+        Self {
+            request_id,
+            buttons: Default::default(),
+        }
+    }
+}
+
+impl EventHandler for DeleteRequestModal {
+    fn update(
+        &mut self,
+        context: &mut UpdateContext,
+        event: Event,
+    ) -> Option<Event> {
+        event.opt().emitted(self.buttons.to_emitter(), |button| {
+            // Do the delete here because we have access to the request store
+            if button == ConfirmButton::Yes {
+                context
+                    .request_store
+                    .delete_request(self.request_id)
+                    .reported(&ViewContext::messages_tx());
+                ViewContext::push_event(Event::HttpSelectRequest(None));
+            }
+            self.close(true);
+        })
+    }
+
+    fn children(&mut self) -> Vec<Component<Child<'_>>> {
+        vec![self.buttons.to_child_mut()]
+    }
+}
+
+impl Modal for DeleteRequestModal {
+    fn title(&self) -> Line<'_> {
+        "Delete Request?".into()
+    }
+
+    fn dimensions(&self) -> (Constraint, Constraint) {
+        (
+            Constraint::Length(ConfirmModal::MIN_WIDTH),
+            Constraint::Length(1),
+        )
+    }
+}
+
+impl Draw for DeleteRequestModal {
+    fn draw(&self, frame: &mut Frame, (): (), metadata: DrawMetadata) {
+        self.buttons.draw(frame, (), metadata.area(), true);
+    }
+}
+
+/// Confirmation modal to delete all requests for a recipe
+#[derive(Debug)]
+pub struct DeleteRecipeRequestsModal {
+    /// Currently selected profile. May be used for the profile filter,
+    /// depending on what the user selects
+    profile_id: Option<ProfileId>,
+    recipe_id: RecipeId,
+    buttons: Component<ButtonGroup<DeleteRecipeRequestsButton>>,
+}
+
+impl DeleteRecipeRequestsModal {
+    pub fn new(profile_id: Option<ProfileId>, recipe_id: RecipeId) -> Self {
+        Self {
+            profile_id,
+            recipe_id,
+            buttons: Default::default(),
+        }
+    }
+}
+
+impl EventHandler for DeleteRecipeRequestsModal {
+    fn update(
+        &mut self,
+        context: &mut UpdateContext,
+        event: Event,
+    ) -> Option<Event> {
+        event.opt().emitted(self.buttons.to_emitter(), |button| {
+            // Do the delete here because we have access to the request store
+            let profile_filter = match button {
+                DeleteRecipeRequestsButton::No => None,
+                DeleteRecipeRequestsButton::Profile => {
+                    Some(self.profile_id.as_ref().into())
+                }
+                DeleteRecipeRequestsButton::All => Some(ProfileFilter::All),
+            };
+            if let Some(profile_filter) = profile_filter {
+                context
+                    .request_store
+                    .delete_recipe_requests(profile_filter, &self.recipe_id)
+                    .reported(&ViewContext::messages_tx());
+                ViewContext::push_event(Event::HttpSelectRequest(None));
+            }
+            self.close(true);
+        })
+    }
+
+    fn children(&mut self) -> Vec<Component<Child<'_>>> {
+        vec![self.buttons.to_child_mut()]
+    }
+}
+
+impl Modal for DeleteRecipeRequestsModal {
+    fn title(&self) -> Line<'_> {
+        format!("Delete Requests for {}?", self.recipe_id).into()
+    }
+
+    fn dimensions(&self) -> (Constraint, Constraint) {
+        const MIN_WIDTH: u16 = 44; // Enough room for the buttons
+        (
+            Constraint::Length(
+                MIN_WIDTH.max((self.title().width() + 4) as u16),
+            ),
+            Constraint::Length(1),
+        )
+    }
+}
+
+impl Draw for DeleteRecipeRequestsModal {
+    fn draw(&self, frame: &mut Frame, (): (), metadata: DrawMetadata) {
+        self.buttons.draw(frame, (), metadata.area(), true);
+    }
+}
+
+/// Buttons for [DeleteRecipeRequestsModal]
+#[derive(
+    Copy, Clone, Debug, Default, Display, EnumCount, EnumIter, PartialEq,
+)]
+enum DeleteRecipeRequestsButton {
+    No,
+    /// Delete requests only for the current profile
+    #[default]
+    #[display("For this profile")]
+    Profile,
+    /// Delete all requests for all profiles
+    #[display("For all profiles")]
+    All,
+}
+
 /// Show most recent notification with timestamp
 #[derive(Debug)]
 pub struct NotificationText {
@@ -364,7 +526,7 @@ impl NotificationText {
 }
 
 impl Draw for NotificationText {
-    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
+    fn draw(&self, frame: &mut Frame, (): (), metadata: DrawMetadata) {
         frame.render_widget(
             Paragraph::new(self.notification.generate()),
             metadata.area(),
