@@ -1,10 +1,11 @@
 //! Components for the "primary" view, which is the paned request/response view
 
 use crate::{
-    http::RequestState,
+    http::{RequestState, RequestStateType},
     message::{Message, RequestConfig},
     util::ResultReported,
     view::{
+        Component, ViewContext,
         common::{
             actions::{IntoMenuAction, MenuAction},
             modal::Modal,
@@ -12,32 +13,35 @@ use crate::{
         component::{
             exchange_pane::{ExchangePane, ExchangePaneEvent},
             help::HelpModal,
+            misc::DeleteRecipeRequestsModal,
             profile_select::ProfilePane,
             recipe_list::{RecipeListPane, RecipeListPaneEvent},
-            recipe_pane::{RecipePane, RecipePaneEvent, RecipePaneProps},
+            recipe_pane::{
+                RecipeMenuAction, RecipePane, RecipePaneEvent, RecipePaneProps,
+            },
         },
         context::UpdateContext,
         draw::{Draw, DrawMetadata},
         event::{Child, Emitter, Event, EventHandler, OptionEvent, ToEmitter},
         state::{
+            StateCell,
             fixed_select::FixedSelectState,
             select::{SelectStateEvent, SelectStateEventType},
         },
         util::persistence::{Persisted, PersistedLazy},
-        Component, ViewContext,
     },
 };
 use derive_more::Display;
-use persisted::SingletonKey;
 use ratatui::{
+    Frame,
     layout::Layout,
     prelude::{Constraint, Rect},
-    Frame,
 };
 use serde::{Deserialize, Serialize};
 use slumber_config::Action;
-use slumber_core::collection::{
-    Collection, ProfileId, RecipeId, RecipeNodeType,
+use slumber_core::{
+    collection::{Collection, ProfileId, RecipeId, RecipeNodeType},
+    http::RequestId,
 };
 use strum::{EnumCount, EnumIter, IntoEnumIterator};
 
@@ -45,36 +49,32 @@ use strum::{EnumCount, EnumIter, IntoEnumIterator};
 #[derive(Debug)]
 pub struct PrimaryView {
     // Own state
-    selected_pane:
-        PersistedLazy<SingletonKey<PrimaryPane>, FixedSelectState<PrimaryPane>>,
+    selected_pane: PersistedLazy<PrimaryPaneKey, FixedSelectState<PrimaryPane>>,
     fullscreen_mode: Persisted<FullscreenModeKey>,
 
     // Children
     profile_pane: Component<ProfilePane>,
     recipe_list_pane: Component<RecipeListPane>,
     recipe_pane: Component<RecipePane>,
-    exchange_pane: Component<ExchangePane>,
+    /// The exchange pane shows a particular request/response. The entire
+    /// component is rebuilt whenever the selected request changes. The key is
+    /// `None` if the recipe list is empty or a folder is selected
+    exchange_pane: StateCell<
+        Option<(RequestId, RequestStateType)>,
+        Component<ExchangePane>,
+    >,
 
     global_actions_emitter: Emitter<GlobalMenuAction>,
 }
 
 impl PrimaryView {
-    pub fn new(
-        collection: &Collection,
-        selected_request: Option<&RequestState>,
-    ) -> Self {
+    pub fn new(collection: &Collection) -> Self {
         let profile_pane = ProfilePane::new(collection);
         let recipe_list_pane = RecipeListPane::new(&collection.recipes);
-        let exchange_pane = ExchangePane::new(
-            selected_request,
-            recipe_list_pane
-                .selected_node()
-                .map(|(_, node_type)| node_type),
-        );
 
         Self {
             selected_pane: PersistedLazy::new(
-                SingletonKey::default(),
+                Default::default(),
                 FixedSelectState::builder()
                     .subscribe([SelectStateEventType::Select])
                     .build(),
@@ -84,7 +84,7 @@ impl PrimaryView {
             recipe_list_pane: recipe_list_pane.into(),
             profile_pane: profile_pane.into(),
             recipe_pane: Default::default(),
-            exchange_pane: exchange_pane.into(),
+            exchange_pane: Default::default(),
 
             global_actions_emitter: Default::default(),
         }
@@ -110,22 +110,6 @@ impl PrimaryView {
         self.profile_pane.data().selected_profile_id()
     }
 
-    /// Set the state of the currently selected request. Call whenever a new
-    /// request is selected, or the selected request changes state
-    pub fn set_request_state(
-        &mut self,
-        selected_request: Option<&RequestState>,
-    ) {
-        self.exchange_pane = ExchangePane::new(
-            selected_request,
-            self.recipe_list_pane
-                .data()
-                .selected_node()
-                .map(|(_, node_type)| node_type),
-        )
-        .into();
-    }
-
     /// Get a definition of the request that should be sent from the current
     /// recipe settings
     pub fn request_config(&self) -> Option<RequestConfig> {
@@ -229,6 +213,27 @@ impl PrimaryView {
     fn send_request(&self) {
         ViewContext::send_message(Message::HttpBeginRequest);
     }
+
+    /// Handle a menu action from the recipe list or recipe pane
+    fn handle_recipe_menu_action(&self, action: RecipeMenuAction) {
+        match action {
+            RecipeMenuAction::CopyUrl => {
+                ViewContext::send_message(Message::CopyRequestUrl);
+            }
+            RecipeMenuAction::CopyCurl => {
+                ViewContext::send_message(Message::CopyRequestCurl);
+            }
+            RecipeMenuAction::DeleteRecipe => {
+                if let Some(recipe_id) = self.selected_recipe_id() {
+                    DeleteRecipeRequestsModal::new(
+                        self.selected_profile_id().cloned(),
+                        recipe_id.clone(),
+                    )
+                    .open();
+                }
+            }
+        }
+    }
 }
 
 impl EventHandler for PrimaryView {
@@ -244,27 +249,27 @@ impl EventHandler for PrimaryView {
 
                 // Pane hotkeys
                 Action::SelectProfileList => {
-                    self.profile_pane.data_mut().open_modal()
+                    self.profile_pane.data_mut().open_modal();
                 }
                 Action::SelectRecipeList => self
                     .selected_pane
                     .get_mut()
                     .select(&PrimaryPane::RecipeList),
                 Action::SelectRecipe => {
-                    self.selected_pane.get_mut().select(&PrimaryPane::Recipe)
+                    self.selected_pane.get_mut().select(&PrimaryPane::Recipe);
                 }
                 Action::SelectResponse => {
-                    self.selected_pane.get_mut().select(&PrimaryPane::Exchange)
+                    self.selected_pane.get_mut().select(&PrimaryPane::Exchange);
                 }
 
                 // Toggle fullscreen
                 Action::Fullscreen => {
                     match self.selected_pane.selected() {
                         PrimaryPane::Recipe => {
-                            self.toggle_fullscreen(FullscreenMode::Recipe)
+                            self.toggle_fullscreen(FullscreenMode::Recipe);
                         }
                         PrimaryPane::Exchange => {
-                            self.toggle_fullscreen(FullscreenMode::Exchange)
+                            self.toggle_fullscreen(FullscreenMode::Exchange);
                         }
                         // This isn't fullscreenable. Still consume the event
                         // though, no one else will need it anyway
@@ -289,22 +294,32 @@ impl EventHandler for PrimaryView {
                         .get_mut()
                         .select(&PrimaryPane::RecipeList);
                 }
+                // Menu action forwarded up
+                RecipeListPaneEvent::Action(action) => {
+                    self.handle_recipe_menu_action(action);
+                }
             })
             .emitted(self.recipe_pane.to_emitter(), |event| match event {
                 RecipePaneEvent::Click => {
                     self.selected_pane.get_mut().select(&PrimaryPane::Recipe);
                 }
+                RecipePaneEvent::Action(action) => {
+                    self.handle_recipe_menu_action(action);
+                }
             })
-            .emitted(self.exchange_pane.to_emitter(), |event| match event {
-                ExchangePaneEvent::Click => {
-                    self.selected_pane.get_mut().select(&PrimaryPane::Exchange)
+            .emitted(self.exchange_pane.borrow().to_emitter(), |event| {
+                match event {
+                    ExchangePaneEvent::Click => self
+                        .selected_pane
+                        .get_mut()
+                        .select(&PrimaryPane::Exchange),
                 }
             })
             .emitted(self.global_actions_emitter, |menu_action| {
                 // Handle our own menu action type
                 match menu_action {
                     GlobalMenuAction::EditCollection => {
-                        ViewContext::send_message(Message::CollectionEdit)
+                        ViewContext::send_message(Message::CollectionEdit);
                     }
                 }
             })
@@ -321,13 +336,18 @@ impl EventHandler for PrimaryView {
             self.profile_pane.to_child_mut(),
             self.recipe_list_pane.to_child_mut(),
             self.recipe_pane.to_child_mut(),
-            self.exchange_pane.to_child_mut(),
+            self.exchange_pane.get_mut().to_child_mut(),
         ]
     }
 }
 
-impl Draw for PrimaryView {
-    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
+impl<'a> Draw<PrimaryViewProps<'a>> for PrimaryView {
+    fn draw(
+        &self,
+        frame: &mut Frame,
+        props: PrimaryViewProps<'a>,
+        metadata: DrawMetadata,
+    ) {
         // We draw all panes regardless of fullscreen state, so they can run
         // their necessary state updates. We just give the hidden panes an empty
         // rect to draw into so they don't appear at all
@@ -367,7 +387,24 @@ impl Draw for PrimaryView {
             panes.recipe.focus,
         );
 
-        self.exchange_pane.draw(
+        // Rebuild the exchange pane whenever we select a new request or the
+        // current request transitions between states
+        let exchange_pane = self.exchange_pane.get_or_update(
+            &props.selected_request.map(|request_state| {
+                (request_state.id(), request_state.into())
+            }),
+            || {
+                ExchangePane::new(
+                    props.selected_request,
+                    self.recipe_list_pane
+                        .data()
+                        .selected_node()
+                        .map(|(_, node_type)| node_type),
+                )
+                .into()
+            },
+        );
+        exchange_pane.draw(
             frame,
             (),
             panes.exchange.area,
@@ -376,6 +413,16 @@ impl Draw for PrimaryView {
     }
 }
 
+#[derive(Debug, Default)]
+pub struct PrimaryViewProps<'a> {
+    pub selected_request: Option<&'a RequestState>,
+}
+
+/// Persistence key for selected pane
+#[derive(Debug, Default, persisted::PersistedKey, Serialize)]
+#[persisted(PrimaryPane)]
+struct PrimaryPaneKey;
+
 /// Selectable panes in the primary view mode
 #[derive(
     Copy,
@@ -396,6 +443,11 @@ enum PrimaryPane {
     Exchange,
 }
 
+/// Persistence key for fullscreen mode
+#[derive(Debug, Default, persisted::PersistedKey, Serialize)]
+#[persisted(Option<FullscreenMode>)]
+struct FullscreenModeKey;
+
 /// Panes that can be fullscreened. This is separate from [PrimaryPane] because
 /// it makes it easy to check when we should exit fullscreen mode.
 #[derive(Copy, Clone, Debug, PartialEq, Serialize, Deserialize)]
@@ -406,11 +458,6 @@ enum FullscreenMode {
     Exchange,
 }
 
-/// Persistence key for fullscreen mode
-#[derive(Debug, Default, persisted::PersistedKey, Serialize)]
-#[persisted(Option<FullscreenMode>)]
-struct FullscreenModeKey;
-
 /// Menu actions available in all contexts
 #[derive(Copy, Clone, Debug, Display, EnumIter)]
 enum GlobalMenuAction {
@@ -440,7 +487,7 @@ mod tests {
     use super::*;
     use crate::{
         message::{Message, RequestConfig},
-        test_util::{harness, terminal, TestHarness, TestTerminal},
+        test_util::{TestHarness, TestTerminal, harness, terminal},
         view::{
             test_util::TestComponent, util::persistence::DatabasePersistedStore,
         },
@@ -448,14 +495,15 @@ mod tests {
     use crossterm::event::KeyCode;
     use persisted::PersistedStore;
     use rstest::rstest;
-    use slumber_core::{assert_matches, http::BuildOptions};
+    use slumber_core::http::BuildOptions;
+    use slumber_util::assert_matches;
 
     /// Create component to be tested
     fn create_component<'term>(
         harness: &mut TestHarness,
         terminal: &'term TestTerminal,
-    ) -> TestComponent<'term, PrimaryView, ()> {
-        let view = PrimaryView::new(&harness.collection, None);
+    ) -> TestComponent<'term, PrimaryView> {
+        let view = PrimaryView::new(&harness.collection);
         let mut component = TestComponent::new(harness, terminal, view);
         // Initial events
         assert_matches!(
@@ -471,7 +519,7 @@ mod tests {
     #[rstest]
     fn test_pane_persistence(mut harness: TestHarness, terminal: TestTerminal) {
         DatabasePersistedStore::store_persisted(
-            &SingletonKey::<PrimaryPane>::default(),
+            &PrimaryPaneKey,
             &PrimaryPane::Exchange,
         );
         DatabasePersistedStore::store_persisted(
