@@ -4,6 +4,7 @@ use crate::{
     context::TuiContext,
     util::{run_command, spawn},
     view::{
+        Component, IntoModal, ViewContext,
         common::{
             modal::Modal,
             text_box::{TextBox, TextBoxEvent, TextBoxProps},
@@ -14,20 +15,19 @@ use crate::{
         event::{Child, Emitter, Event, EventHandler, OptionEvent, ToEmitter},
         state::Identified,
         util::{highlight, str_to_text},
-        Component, IntoModal, ViewContext,
     },
 };
 use anyhow::Context;
 use bytes::Bytes;
 use persisted::PersistedContainer;
 use ratatui::{
+    Frame,
     layout::{Constraint, Layout},
     text::Text,
-    Frame,
 };
 use slumber_config::Action;
 use slumber_core::{
-    http::{content_type::ContentType, ResponseBody, ResponseRecord},
+    http::{ResponseBody, ResponseRecord, content_type::ContentType},
     util::MaybeStr,
 };
 use std::{borrow::Cow, mem, sync::Arc};
@@ -82,8 +82,6 @@ impl QueryableBody {
             ))
             .placeholder_focused("Enter query command (ex: `jq .results`)")
             .default_value(default_query.clone().unwrap_or_default());
-        // Don't use a debounce on this one, because we don't want to
-        // auto-execute commands that will have a side effect
         let export_text_box = TextBox::default().placeholder_focused(
             "Enter export command (ex: `tee > response.json`)",
         );
@@ -166,7 +164,7 @@ impl QueryableBody {
             let emitter = self.emitter;
             let abort_handle =
                 self.spawn_command(command, body, move |_, result| {
-                    emitter.emit(QueryComplete(result))
+                    emitter.emit(QueryComplete(result));
                 });
             self.query_state = QueryState::Running(abort_handle);
         }
@@ -280,7 +278,7 @@ impl EventHandler for QueryableBody {
 }
 
 impl Draw for QueryableBody {
-    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
+    fn draw(&self, frame: &mut Frame, (): (), metadata: DrawMetadata) {
         let [body_area, query_area] =
             Layout::vertical([Constraint::Min(0), Constraint::Length(1)])
                 .areas(metadata.area());
@@ -402,7 +400,8 @@ impl TextState {
             // separate task because it's generally very fast. If this is slow
             // enough that it affects the user, the "large" body size is
             // probably too low
-            // 2024 edition: if-let chain
+            // unstable: if-let chain
+            // https://github.com/rust-lang/rust/pull/132833
             let (text, pretty): (Cow<str>, bool) = match content_type {
                 Some(content_type) if prettify => content_type
                     .prettify(text)
@@ -474,7 +473,7 @@ mod tests {
     use super::*;
     use crate::{
         context::TuiContext,
-        test_util::{harness, run_local, terminal, TestHarness, TestTerminal},
+        test_util::{TestHarness, TestTerminal, harness, run_local, terminal},
         view::{
             test_util::TestComponent,
             util::persistence::{DatabasePersistedStore, PersistedLazy},
@@ -485,11 +484,8 @@ mod tests {
     use ratatui::{layout::Margin, text::Span};
     use rstest::{fixture, rstest};
     use serde::Serialize;
-    use slumber_core::{
-        assert_matches,
-        http::{ResponseBody, ResponseRecord},
-        test_util::{temp_dir, Factory, TempDir},
-    };
+    use slumber_core::http::{ResponseBody, ResponseRecord};
+    use slumber_util::{Factory, TempDir, assert_matches, temp_dir};
     use tokio::fs;
 
     const TEXT: &str = "{\"greeting\":\"hello\"}";
@@ -551,8 +547,8 @@ mod tests {
 
         // Type something into the query box
         component.int().send_key(KeyCode::Char('/')).assert_empty();
-        // Both the debounce and the subprocess use local tasks, so we need to
-        // run in a local set. When this future exits, all tasks are done
+        // The subprocess uses local tasks, so we need to run in a local set.
+        // When this future exits, all tasks are done
         run_local(async {
             component
                 .int()
@@ -572,15 +568,11 @@ mod tests {
 
         // Cancelling out of the text box should reset the query value
         component.int().send_key(KeyCode::Char('/')).assert_empty();
-        run_local(async {
-            // Local task needed for the debounce
-            component
-                .int()
-                .send_text("more text")
-                .send_key(KeyCode::Esc)
-                .assert_empty();
-        })
-        .await;
+        component
+            .int()
+            .send_text("more text")
+            .send_key(KeyCode::Esc)
+            .assert_empty();
         let data = component.data();
         assert_eq!(data.last_executed_query.as_deref(), Some("head -c 1"));
         assert_eq!(data.query_text_box.data().text(), "head -c 1");
@@ -639,8 +631,8 @@ mod tests {
         terminal: TestTerminal,
         response: Arc<ResponseRecord>,
     ) {
-        // Setting initial value triggers a debounce event
-        let mut component = run_local(async {
+        // Local task is spawned to execute the initial subprocess
+        let component = run_local(async {
             TestComponent::new(
                 &harness,
                 &terminal,
@@ -649,9 +641,6 @@ mod tests {
         })
         .await;
 
-        // After the debounce, there's a change event that spawns the command
-        run_local(async { component.int().drain_draw().assert_empty() }).await;
-
         assert_eq!(
             component.data().last_executed_query.as_deref(),
             Some("head -n 1")
@@ -667,10 +656,10 @@ mod tests {
         terminal: TestTerminal,
         response: Arc<ResponseRecord>,
     ) {
-        DatabasePersistedStore::store_persisted(&Key, &"".to_owned());
+        DatabasePersistedStore::store_persisted(&Key, &String::new());
 
-        // Setting initial value triggers a debounce event
-        let mut component = run_local(async {
+        // Local task is spawned to execute the initial subprocess
+        let component = run_local(async {
             TestComponent::new(
                 &harness,
                 &terminal,
@@ -683,9 +672,6 @@ mod tests {
         })
         .await;
 
-        // After the debounce, there's a change event that spawns the command
-        run_local(async { component.int().drain_draw().assert_empty() }).await;
-
         assert_eq!(
             component.data().last_executed_query.as_deref(),
             Some("head -n 1")
@@ -701,7 +687,6 @@ mod tests {
         response: Arc<ResponseRecord>,
         temp_dir: TempDir,
     ) {
-        // Setting initial value triggers a debounce event
         let mut component = TestComponent::builder(
             &harness,
             &terminal,
@@ -724,7 +709,7 @@ mod tests {
             .assert_empty();
         // Triggers the background task
         run_local(async {
-            component.int().send_key(KeyCode::Enter).assert_empty()
+            component.int().send_key(KeyCode::Enter).assert_empty();
         })
         .await;
         // Success should push a notification
@@ -738,7 +723,7 @@ mod tests {
         // Error should appear as a modal
         component.int().send_text(":bad!").assert_empty();
         run_local(async {
-            component.int().send_key(KeyCode::Enter).assert_empty()
+            component.int().send_key(KeyCode::Enter).assert_empty();
         })
         .await;
         component.int().drain_draw().assert_empty();
