@@ -1,27 +1,27 @@
 use crate::{
     context::TuiContext,
-    message::Message,
     view::{
+        Component, ViewContext,
         common::{
+            Pane,
             actions::{IntoMenuAction, MenuAction},
             list::List,
             text_box::{TextBox, TextBoxEvent, TextBoxProps},
-            Pane,
         },
+        component::recipe_pane::RecipeMenuAction,
         context::UpdateContext,
         draw::{Draw, DrawMetadata, Generate},
         event::{Child, Emitter, Event, EventHandler, OptionEvent, ToEmitter},
         state::select::{SelectState, SelectStateEvent, SelectStateEventType},
         util::persistence::{Persisted, PersistedLazy},
-        Component, ViewContext,
     },
 };
 use derive_more::{Deref, DerefMut};
-use persisted::{PersistedKey, SingletonKey};
+use persisted::PersistedKey;
 use ratatui::{
+    Frame,
     layout::{Constraint, Layout},
     text::Text,
-    Frame,
 };
 use serde::{Deserialize, Serialize};
 use slumber_config::Action;
@@ -29,7 +29,7 @@ use slumber_core::collection::{
     HasId, RecipeId, RecipeLookupKey, RecipeNode, RecipeNodeType, RecipeTree,
 };
 use std::collections::HashSet;
-use strum::{EnumIter, IntoEnumIterator};
+use strum::IntoEnumIterator;
 
 /// List/tree of recipes and folders. This is mostly just a list, but with some
 /// extra logic to allow expanding/collapsing nodes. This could be made into a
@@ -43,10 +43,10 @@ use strum::{EnumIter, IntoEnumIterator};
 /// implementation.
 #[derive(Debug)]
 pub struct RecipeListPane {
-    /// Emitter for the on-click event, to focus the pane
-    click_emitter: Emitter<RecipeListPaneEvent>,
+    /// Emitter for events that the parent will consume
+    emitter: Emitter<RecipeListPaneEvent>,
     /// Emitter for menu actions, to be handled by our parent
-    actions_emitter: Emitter<RecipeListMenuAction>,
+    actions_emitter: Emitter<RecipeMenuAction>,
     /// The visible list of items is tracked using normal list state, so we can
     /// easily re-use existing logic. We'll rebuild this any time a folder is
     /// expanded/collapsed (i.e whenever the list of items changes)
@@ -60,7 +60,7 @@ pub struct RecipeListPane {
     /// (if they were collapsed at the time of deletion). That isn't really an
     /// issue though, it just means it'll be pre-collapsed if the user ever
     /// adds the folder back. Not worth working around.
-    collapsed: Persisted<SingletonKey<Collapsed>>,
+    collapsed: Persisted<CollapsedKey>,
 
     filter: Component<TextBox>,
     filter_focused: bool,
@@ -73,8 +73,7 @@ impl RecipeListPane {
 
         // This clone is unfortunate, but we can't hold onto a reference to the
         // recipes
-        let collapsed: Persisted<SingletonKey<Collapsed>> =
-            Persisted::default();
+        let collapsed: Persisted<CollapsedKey> = Persisted::default();
         let select = PersistedLazy::new(
             SelectedRecipeKey,
             collapsed.build_select_state(recipes, ""),
@@ -82,7 +81,7 @@ impl RecipeListPane {
         let filter =
             TextBox::default().placeholder(format!("{binding} to filter"));
         Self {
-            click_emitter: Default::default(),
+            emitter: Default::default(),
             actions_emitter: Default::default(),
             select: select.into(),
             collapsed,
@@ -100,6 +99,16 @@ impl RecipeListPane {
             .map(|node| (&node.id, node.kind))
     }
 
+    /// Get the ID of the selected recipe, if a node is selected and it's a
+    /// recipe
+    fn selected_recipe_id(&self) -> Option<&RecipeId> {
+        self.select
+            .data()
+            .selected()
+            .filter(|node| node.is_recipe())
+            .map(|node| &node.id)
+    }
+
     /// Set the currently selected folder as expanded/collapsed (or toggle it).
     /// If a folder is not selected, do nothing. Returns whether a change was
     /// made.
@@ -159,7 +168,7 @@ impl EventHandler for RecipeListPane {
             .opt()
             .action(|action, propagate| match action {
                 Action::LeftClick => {
-                    self.click_emitter.emit(RecipeListPaneEvent::Click)
+                    self.emitter.emit(RecipeListPaneEvent::Click);
                 }
                 Action::Left => {
                     self.set_selected_collapsed(CollapseState::Collapse);
@@ -188,21 +197,17 @@ impl EventHandler for RecipeListPane {
                 TextBoxEvent::Focus => self.filter_focused = true,
                 TextBoxEvent::Change => self.rebuild_select_state(),
                 TextBoxEvent::Cancel | TextBoxEvent::Submit => {
-                    self.filter_focused = false
+                    self.filter_focused = false;
                 }
             })
-            .emitted(self.actions_emitter, |menu_action| match menu_action {
-                RecipeListMenuAction::CopyUrl => {
-                    ViewContext::send_message(Message::CopyRequestUrl)
-                }
-                RecipeListMenuAction::CopyCurl => {
-                    ViewContext::send_message(Message::CopyRequestCurl)
-                }
+            .emitted(self.actions_emitter, |menu_action| {
+                // Forward this to our parent
+                self.emitter.emit(RecipeListPaneEvent::Action(menu_action));
             })
     }
 
     fn menu_actions(&self) -> Vec<MenuAction> {
-        RecipeListMenuAction::iter()
+        RecipeMenuAction::iter()
             .map(MenuAction::with_data(self, self.actions_emitter))
             .collect()
     }
@@ -215,7 +220,7 @@ impl EventHandler for RecipeListPane {
 }
 
 impl Draw for RecipeListPane {
-    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
+    fn draw(&self, frame: &mut Frame, (): (), metadata: DrawMetadata) {
         let context = TuiContext::get();
 
         let title = context
@@ -252,7 +257,7 @@ impl Draw for RecipeListPane {
 /// Notify parent when this pane is clicked
 impl ToEmitter<RecipeListPaneEvent> for RecipeListPane {
     fn to_emitter(&self) -> Emitter<RecipeListPaneEvent> {
-        self.click_emitter
+        self.emitter
     }
 }
 
@@ -264,27 +269,18 @@ struct SelectedRecipeKey;
 /// Emitted event type for the recipe list pane
 #[derive(Debug)]
 pub enum RecipeListPaneEvent {
+    /// Pane was clicked; focus it
     Click,
+    /// Forward menu actions to the parent because it has the needed context
+    Action(RecipeMenuAction),
 }
 
-/// Items in the actions popup menu
-#[derive(Copy, Clone, Debug, derive_more::Display, EnumIter)]
-enum RecipeListMenuAction {
-    #[display("Copy URL")]
-    CopyUrl,
-    #[display("Copy as cURL")]
-    CopyCurl,
-}
-
-impl IntoMenuAction<RecipeListPane> for RecipeListMenuAction {
+impl IntoMenuAction<RecipeListPane> for RecipeMenuAction {
     fn enabled(&self, data: &RecipeListPane) -> bool {
-        let recipe = data
-            .select
-            .data()
-            .selected()
-            .filter(|node| node.is_recipe());
+        let has_recipe = data.selected_recipe_id().is_some();
+        // Use a match so we have to think about this for any new variants
         match self {
-            Self::CopyUrl | Self::CopyCurl => recipe.is_some(),
+            Self::CopyUrl | Self::CopyCurl | Self::DeleteRecipe => has_recipe,
         }
     }
 }
@@ -338,8 +334,9 @@ impl PartialEq<RecipeListItem> for RecipeId {
     }
 }
 
-impl<'a> Generate for &'a RecipeListItem {
-    type Output<'this> = Text<'this>
+impl Generate for &RecipeListItem {
+    type Output<'this>
+        = Text<'this>
     where
         Self: 'this;
 
@@ -364,6 +361,11 @@ impl<'a> Generate for &'a RecipeListItem {
     }
 }
 
+/// Persistence key for collapsed state
+#[derive(Debug, Default, persisted::PersistedKey, Serialize)]
+#[persisted(Collapsed)]
+struct CollapsedKey;
+
 /// Set of collapsed folders. Newtype allows us to encapsulate some extra
 /// functionality
 #[derive(Debug, Default, Deref, DerefMut, Serialize, Deserialize)]
@@ -446,17 +448,17 @@ impl Collapsed {
 mod tests {
     use super::*;
     use crate::{
-        test_util::{terminal, TestHarness, TestTerminal},
+        test_util::{TestHarness, TestTerminal, terminal},
         view::test_util::TestComponent,
     };
     use crossterm::event::KeyCode;
     use itertools::Itertools;
     use rstest::{fixture, rstest};
     use slumber_core::{
-        assert_matches,
         collection::{Collection, Recipe},
-        test_util::{by_id, Factory},
+        test_util::by_id,
     };
+    use slumber_util::{Factory, assert_matches};
 
     /// Test the filter box
     #[rstest]
