@@ -8,48 +8,47 @@ pub use persistence::RecipeOverrideStore;
 
 use crate::{
     context::TuiContext,
-    message::{Message, RequestConfig},
+    message::RequestConfig,
     view::{
+        Component,
         common::{
-            actions::{IntoMenuAction, MenuAction},
             Pane,
+            actions::{IntoMenuAction, MenuAction},
         },
         component::recipe_pane::recipe::RecipeDisplay,
         context::UpdateContext,
         draw::{Draw, DrawMetadata, Generate},
         event::{Child, Emitter, Event, EventHandler, OptionEvent, ToEmitter},
         state::StateCell,
-        Component, ViewContext,
     },
 };
 use derive_more::Display;
 use itertools::{Itertools, Position};
 use ratatui::{
-    text::{Line, Text},
     Frame,
+    text::{Line, Text},
 };
 use slumber_config::Action;
 use slumber_core::{
     collection::{Folder, HasId, ProfileId, RecipeId, RecipeNode},
     util::doc_link,
 };
-use std::cell::Ref;
 use strum::{EnumIter, IntoEnumIterator};
 
 /// Display for the current recipe node, which could be a recipe, a folder, or
 /// empty
 #[derive(Debug, Default)]
 pub struct RecipePane {
-    /// Emitter for the on-click event, to focus the pane
-    click_emitter: Emitter<RecipePaneEvent>,
+    /// Emitter for events that the parent will consume
+    emitter: Emitter<RecipePaneEvent>,
     /// Emitter for menu actions, to be handled by our parent
-    actions_emitter: Emitter<RecipePaneMenuAction>,
+    actions_emitter: Emitter<RecipeMenuAction>,
     /// All UI state derived from the recipe is stored together, and reset when
     /// the recipe or profile changes
     recipe_state: StateCell<RecipeStateKey, Component<Option<RecipeDisplay>>>,
 }
 
-#[derive(Clone)]
+#[derive(Debug)]
 pub struct RecipePaneProps<'a> {
     /// ID of the recipe *or* folder selected
     pub selected_recipe_node: Option<&'a RecipeNode>,
@@ -60,14 +59,14 @@ impl RecipePane {
     /// Get a definition of the request that should be sent from the current
     /// recipe settings
     pub fn request_config(&self) -> Option<RequestConfig> {
-        let state_key = self.recipe_state.get_key()?;
+        let state_key = self.recipe_state.borrow_key();
         let recipe_id = state_key.recipe_id.clone()?;
         let profile_id = state_key.selected_profile_id.clone();
-        let recipe_state = self.recipe_state.get()?;
+        let recipe_state = self.recipe_state.borrow();
         let options = recipe_state.data().as_ref()?.build_options();
         Some(RequestConfig {
-            recipe_id,
             profile_id,
+            recipe_id,
             options,
         })
     }
@@ -78,33 +77,22 @@ impl EventHandler for RecipePane {
         event
             .opt()
             .action(|action, propagate| match action {
-                Action::LeftClick => {
-                    self.click_emitter.emit(RecipePaneEvent::Click)
-                }
+                Action::LeftClick => self.emitter.emit(RecipePaneEvent::Click),
                 _ => propagate.set(),
             })
-            .emitted(self.actions_emitter, |menu_action| match menu_action {
-                RecipePaneMenuAction::CopyUrl => {
-                    ViewContext::send_message(Message::CopyRequestUrl)
-                }
-                RecipePaneMenuAction::CopyCurl => {
-                    ViewContext::send_message(Message::CopyRequestCurl)
-                }
+            .emitted(self.actions_emitter, |menu_action| {
+                self.emitter.emit(RecipePaneEvent::Action(menu_action));
             })
     }
 
     fn menu_actions(&self) -> Vec<MenuAction> {
-        RecipePaneMenuAction::iter()
+        RecipeMenuAction::iter()
             .map(MenuAction::with_data(self, self.actions_emitter))
             .collect()
     }
 
     fn children(&mut self) -> Vec<Component<Child<'_>>> {
-        if let Some(state) = self.recipe_state.get_mut() {
-            vec![state.to_child_mut()]
-        } else {
-            vec![]
-        }
+        vec![self.recipe_state.get_mut().to_child_mut()]
     }
 }
 
@@ -167,56 +155,63 @@ impl<'a> Draw<RecipePaneProps<'a>> for RecipePane {
                 frame.render_widget(folder.generate(), inner_area);
             }
             Some(RecipeNode::Recipe(_)) => {
-                recipe_state.draw_opt(frame, (), inner_area, true)
+                recipe_state.draw_opt(frame, (), inner_area, true);
             }
-        };
+        }
     }
 }
 
 /// Notify parent when this pane is clicked
 impl ToEmitter<RecipePaneEvent> for RecipePane {
     fn to_emitter(&self) -> Emitter<RecipePaneEvent> {
-        self.click_emitter
+        self.emitter
     }
 }
 
 /// Emitted event for the recipe pane component
 #[derive(Debug)]
 pub enum RecipePaneEvent {
+    /// Pane was clicked; focus it
     Click,
+    /// Forward menu actions to the parent because it has the needed context
+    Action(RecipeMenuAction),
 }
 
 /// Template preview state will be recalculated when any of these fields change
-#[derive(Clone, Debug, PartialEq)]
+#[derive(Clone, Debug, Default, PartialEq)]
 struct RecipeStateKey {
     selected_profile_id: Option<ProfileId>,
     recipe_id: Option<RecipeId>,
 }
 
-/// Items in the actions popup menu
+/// Items in the actions popup menu. This is shared with the recipe list and
+/// handled in our parent to deduplicate the logic. Also the parent has access
+/// to needed context for the delete.
 #[derive(Copy, Clone, Debug, Display, EnumIter)]
-enum RecipePaneMenuAction {
+pub enum RecipeMenuAction {
     #[display("Copy URL")]
     CopyUrl,
     #[display("Copy as cURL")]
     CopyCurl,
+    #[display("Delete Requests")]
+    DeleteRecipe,
 }
 
-impl IntoMenuAction<RecipePane> for RecipePaneMenuAction {
+impl IntoMenuAction<RecipePane> for RecipeMenuAction {
     fn enabled(&self, data: &RecipePane) -> bool {
-        let recipe = data.recipe_state.get().and_then(|state| {
-            Ref::filter_map(state, |state| state.data().as_ref()).ok()
-        });
         match self {
-            // Enabled if we have any recipe
-            Self::CopyUrl | Self::CopyCurl => recipe.is_some(),
+            Self::CopyUrl | Self::CopyCurl | Self::DeleteRecipe => {
+                // Enabled if we have any recipe
+                data.recipe_state.borrow().data().is_some()
+            }
         }
     }
 }
 
 /// Render folder as a tree
-impl<'a> Generate for &'a Folder {
-    type Output<'this> = Text<'this>
+impl Generate for &Folder {
+    type Output<'this>
+        = Text<'this>
     where
         Self: 'this;
 
@@ -235,7 +230,6 @@ impl<'a> Generate for &'a Folder {
         // │ └─Get User
         // └─Modify User
 
-        let mut lines = vec![self.name().into()];
         fn add_lines<'a>(
             lines: &mut Vec<Line<'a>>,
             folder: &'a Folder,
@@ -267,6 +261,8 @@ impl<'a> Generate for &'a Folder {
                 }
             }
         }
+
+        let mut lines = vec![self.name().into()];
         add_lines(&mut lines, self, &mut Vec::new());
         lines.into()
     }
@@ -276,10 +272,8 @@ impl<'a> Generate for &'a Folder {
 mod tests {
     use super::*;
     use pretty_assertions::assert_eq;
-    use slumber_core::{
-        collection::Recipe,
-        test_util::{by_id, Factory},
-    };
+    use slumber_core::{collection::Recipe, test_util::by_id};
+    use slumber_util::Factory;
 
     #[test]
     fn test_folder_tree() {
