@@ -2,6 +2,7 @@ use crate::{
     context::TuiContext,
     util::ResultReported,
     view::{
+        ViewContext,
         common::{
             actions::{IntoMenuAction, MenuAction},
             modal::Modal,
@@ -9,20 +10,19 @@ use crate::{
             text_box::TextBox,
         },
         component::{
+            Component,
             misc::TextBoxModal,
             recipe_pane::persistence::{RecipeOverrideKey, RecipeTemplate},
-            Component,
         },
         context::UpdateContext,
         draw::{Draw, DrawMetadata, Generate},
         event::{Child, Emitter, Event, EventHandler, OptionEvent},
         state::fixed_select::FixedSelectState,
-        ViewContext,
     },
 };
 use derive_more::derive::Display;
 use ratatui::{
-    layout::Layout, prelude::Constraint, text::Span, widgets::TableState, Frame,
+    Frame, layout::Layout, prelude::Constraint, text::Span, widgets::TableState,
 };
 use slumber_config::Action;
 use slumber_core::{
@@ -105,7 +105,7 @@ impl EventHandler for AuthenticationDisplay {
             .opt()
             .action(|action, propagate| match action {
                 Action::Edit => {
-                    self.state.open_edit_modal(self.override_emitter)
+                    self.state.open_edit_modal(self.override_emitter);
                 }
                 Action::Reset => self.state.reset_override(),
                 _ => propagate.set(),
@@ -113,12 +113,12 @@ impl EventHandler for AuthenticationDisplay {
             .emitted(
                 self.override_emitter,
                 |SaveAuthenticationOverride(value)| {
-                    self.state.set_override(&value)
+                    self.state.set_override(&value);
                 },
             )
             .emitted(self.actions_emitter, |menu_action| match menu_action {
                 AuthenticationMenuAction::Edit => {
-                    self.state.open_edit_modal(self.override_emitter)
+                    self.state.open_edit_modal(self.override_emitter);
                 }
                 AuthenticationMenuAction::Reset => self.state.reset_override(),
             })
@@ -141,7 +141,7 @@ impl EventHandler for AuthenticationDisplay {
 }
 
 impl Draw for AuthenticationDisplay {
-    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
+    fn draw(&self, frame: &mut Frame, (): (), metadata: DrawMetadata) {
         let styles = &TuiContext::get().styles;
         let [label_area, content_area] =
             Layout::vertical([Constraint::Length(1), Constraint::Min(0)])
@@ -273,10 +273,10 @@ impl State {
                 .validator(|value| value.parse::<Template>().is_ok()),
             move |value| {
                 // Defer the state update into an event, so it can get &mut
-                emitter.emit(SaveAuthenticationOverride(value))
+                emitter.emit(SaveAuthenticationOverride(value));
             },
         )
-        .open()
+        .open();
     }
 
     /// Override the value template for whichever field is selected, and
@@ -346,11 +346,11 @@ enum BasicFields {
 mod tests {
     use super::*;
     use crate::{
-        test_util::{harness, terminal, TestHarness, TestTerminal},
+        test_util::{TestHarness, TestTerminal, harness, terminal},
         view::{
             component::{
-                recipe_pane::persistence::RecipeOverrideValue,
                 RecipeOverrideStore,
+                recipe_pane::persistence::RecipeOverrideValue,
             },
             test_util::TestComponent,
         },
@@ -358,7 +358,7 @@ mod tests {
     use crossterm::event::KeyCode;
     use persisted::PersistedStore;
     use rstest::rstest;
-    use slumber_core::test_util::Factory;
+    use slumber_util::Factory;
 
     #[rstest]
     fn test_edit_basic(harness: TestHarness, terminal: TestTerminal) {
