@@ -1,7 +1,8 @@
 use crate::{
     message::Message,
-    util::{delete_temp_file, temp_file, ResultReported},
+    util::{ResultReported, TempFile},
     view::{
+        Component, ViewContext,
         common::{
             actions::{IntoMenuAction, MenuAction},
             text_window::{ScrollbarMargins, TextWindow, TextWindowProps},
@@ -14,7 +15,6 @@ use crate::{
         draw::{Draw, DrawMetadata},
         event::{Child, Emitter, Event, EventHandler, OptionEvent},
         util::view_text,
-        Component, ViewContext,
     },
 };
 use anyhow::Context;
@@ -27,16 +27,14 @@ use slumber_core::{
     http::content_type::ContentType,
     template::Template,
 };
-use std::{
-    fs,
-    path::{Path, PathBuf},
-};
+use std::fs;
 use strum::{EnumIter, IntoEnumIterator};
 use tracing::debug;
 
 /// Render recipe body. The variant is based on the incoming body type, and
 /// determines the representation
 #[derive(Debug)]
+#[expect(clippy::large_enum_variant)]
 pub enum RecipeBodyDisplay {
     Raw(Component<RawBody>),
     Form(Component<RecipeFieldTable<FormRowKey, FormRowToggleKey>>),
@@ -101,10 +99,10 @@ impl EventHandler for RecipeBodyDisplay {
 }
 
 impl Draw for RecipeBodyDisplay {
-    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
+    fn draw(&self, frame: &mut Frame, (): (), metadata: DrawMetadata) {
         match self {
             RecipeBodyDisplay::Raw(inner) => {
-                inner.draw(frame, (), metadata.area(), true)
+                inner.draw(frame, (), metadata.area(), true);
             }
             RecipeBodyDisplay::Form(form) => form.draw(
                 frame,
@@ -156,37 +154,33 @@ impl RawBody {
     /// the body to a temp file so the editor subprocess can access it. We'll
     /// read it back later.
     fn open_editor(&mut self) {
-        let path = temp_file();
-        debug!(?path, "Writing body to file for editing");
-        let Some(_) =
-            fs::write(&path, self.body.template().display().as_bytes())
-                .with_context(|| {
-                    format!("Error writing body to file {path:?} for editing")
-                })
+        let Some(file) =
+            TempFile::new(self.body.template().display().as_bytes())
                 .reported(&ViewContext::messages_tx())
         else {
             // Write failed
             return;
         };
+        debug!(?file, "Wrote body to file for editing");
 
         let emitter = self.override_emitter;
         ViewContext::send_message(Message::FileEdit {
-            path,
-            on_complete: Box::new(move |path| {
-                emitter.emit(SaveBodyOverride(path))
+            file,
+            on_complete: Box::new(move |file| {
+                emitter.emit(SaveBodyOverride(file));
             }),
-        })
+        });
     }
 
     /// Read the user's edited body from the temp file we created, and rebuild
     /// the body from that
-    fn load_override(&mut self, path: &Path) {
+    fn load_override(&mut self, file: TempFile) {
         // Read the body back from the temp file we handed to the editor, then
         // delete it to prevent cluttering the disk
-        debug!(?path, "Reading edited body from file");
-        let Some(body) = fs::read_to_string(path)
+        debug!(?file, "Reading edited body from file");
+        let Some(body) = fs::read_to_string(file.path())
             .with_context(|| {
-                format!("Error reading edited body from file {path:?}")
+                format!("Error reading edited body from file {:?}", file.path())
             })
             .reported(&ViewContext::messages_tx())
         else {
@@ -195,9 +189,6 @@ impl RawBody {
             return;
         };
 
-        // Clean up after ourselves
-        delete_temp_file(path);
-
         let Some(template) = body
             .parse::<Template>()
             .reported(&ViewContext::messages_tx())
@@ -221,13 +212,13 @@ impl EventHandler for RawBody {
                 Action::Reset => self.body.reset_override(),
                 _ => propagate.set(),
             })
-            .emitted(self.override_emitter, |SaveBodyOverride(path)| {
-                self.load_override(&path)
+            .emitted(self.override_emitter, |SaveBodyOverride(file)| {
+                self.load_override(file);
             })
             .emitted(self.actions_emitter, |menu_action| match menu_action {
                 RawBodyMenuAction::View => self.view_body(),
                 RawBodyMenuAction::Copy => {
-                    ViewContext::send_message(Message::CopyRequestBody)
+                    ViewContext::send_message(Message::CopyRequestBody);
                 }
                 RawBodyMenuAction::Edit => self.open_editor(),
                 RawBodyMenuAction::Reset => self.body.reset_override(),
@@ -246,7 +237,7 @@ impl EventHandler for RawBody {
 }
 
 impl Draw for RawBody {
-    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
+    fn draw(&self, frame: &mut Frame, (): (), metadata: DrawMetadata) {
         let area = metadata.area();
         self.text_window.draw(
             frame,
@@ -312,14 +303,14 @@ impl IntoMenuAction<RawBody> for RawBodyMenuAction {
 /// Local event to save a user's override body. Triggered from the on_complete
 /// callback when the user closes the editor.
 #[derive(Debug)]
-struct SaveBodyOverride(PathBuf);
+struct SaveBodyOverride(TempFile);
 
 #[cfg(test)]
 mod tests {
     use super::*;
     use crate::{
         context::TuiContext,
-        test_util::{harness, terminal, TestHarness, TestTerminal},
+        test_util::{TestHarness, TestTerminal, harness, terminal},
         view::{
             component::recipe_pane::persistence::{
                 RecipeOverrideStore, RecipeOverrideValue,
@@ -331,7 +322,7 @@ mod tests {
     use persisted::PersistedStore;
     use ratatui::{style::Styled, text::Span};
     use rstest::rstest;
-    use slumber_core::{assert_matches, test_util::Factory};
+    use slumber_util::{Factory, assert_matches};
 
     /// Test editing the body, which should open a file for the user to edit,
     /// then load the response
@@ -360,18 +351,18 @@ mod tests {
         // Open the editor
         harness.clear_messages();
         component.int().send_key(KeyCode::Char('e')).assert_empty();
-        let (path, on_complete) = assert_matches!(
+        let (file, on_complete) = assert_matches!(
             harness.pop_message_now(),
             Message::FileEdit {
-                path,
+                file,
                 on_complete,
-            } => (path, on_complete),
+            } => (file, on_complete),
         );
-        assert_eq!(fs::read(&path).unwrap(), b"hello!");
+        assert_eq!(fs::read(file.path()).unwrap(), b"hello!");
 
         // Simulate the editor modifying the file
-        fs::write(&path, "goodbye!").unwrap();
-        on_complete(path);
+        fs::write(file.path(), "goodbye!").unwrap();
+        on_complete(file);
         component.int().drain_draw().assert_empty();
 
         assert_eq!(
