@@ -1,6 +1,6 @@
 //! Single-session persistence for recipe overrides
 
-use crate::view::{common::template_preview::TemplatePreview, ViewContext};
+use crate::view::{ViewContext, common::template_preview::TemplatePreview};
 use persisted::{PersistedContainer, PersistedLazy, PersistedStore};
 use slumber_core::{
     collection::RecipeId, http::content_type::ContentType, template::Template,
@@ -38,7 +38,7 @@ impl PersistedStore<RecipeOverrideKey> for RecipeOverrideStore {
             debug!(?key, ?template, "Persisting recipe override");
             ViewContext::with_override_store_mut(|store| {
                 store.0.insert(key.clone(), template.clone());
-            })
+            });
         }
     }
 }
