@@ -1,4 +1,5 @@
 use crate::view::{
+    Component,
     common::{tabs::Tabs, template_preview::TemplatePreview},
     component::recipe_pane::{
         authentication::AuthenticationDisplay,
@@ -9,11 +10,9 @@ use crate::view::{
     draw::{Draw, DrawMetadata},
     event::{Child, EventHandler},
     util::persistence::PersistedLazy,
-    Component,
 };
 use derive_more::Display;
-use persisted::SingletonKey;
-use ratatui::{layout::Layout, prelude::Constraint, widgets::Paragraph, Frame};
+use ratatui::{Frame, layout::Layout, prelude::Constraint, widgets::Paragraph};
 use serde::{Deserialize, Serialize};
 use slumber_core::{
     collection::{Recipe, RecipeId},
@@ -26,7 +25,7 @@ use strum::{EnumCount, EnumIter};
 /// recreated every time the recipe/profile changes.
 #[derive(Debug)]
 pub struct RecipeDisplay {
-    tabs: Component<PersistedLazy<SingletonKey<Tab>, Tabs<Tab>>>,
+    tabs: Component<PersistedLazy<RecipeTabKey, Tabs<Tab>>>,
     url: TemplatePreview,
     method: HttpMethod,
     query: Component<RecipeFieldTable<QueryRowKey, QueryRowToggleKey>>,
@@ -98,11 +97,9 @@ impl RecipeDisplay {
 
     /// Generate a [BuildOptions] instance based on current UI state
     pub fn build_options(&self) -> BuildOptions {
-        let authentication = self
-            .authentication
-            .data()
-            .as_ref()
-            .and_then(|authentication| authentication.override_value());
+        let authentication = self.authentication.data().as_ref().and_then(
+            super::authentication::AuthenticationDisplay::override_value,
+        );
         let form_fields = self
             .body
             .data()
@@ -118,7 +115,7 @@ impl RecipeDisplay {
             .body
             .data()
             .as_ref()
-            .and_then(|body| body.override_value());
+            .and_then(super::body::RecipeBodyDisplay::override_value);
 
         BuildOptions {
             authentication,
@@ -143,7 +140,7 @@ impl EventHandler for RecipeDisplay {
 }
 
 impl Draw for RecipeDisplay {
-    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
+    fn draw(&self, frame: &mut Frame, (): (), metadata: DrawMetadata) {
         // Render request contents
         let method = self.method.to_string();
 
@@ -189,12 +186,17 @@ impl Draw for RecipeDisplay {
                 true,
             ),
             Tab::Authentication => {
-                self.authentication.draw_opt(frame, (), content_area, true)
+                self.authentication.draw_opt(frame, (), content_area, true);
             }
         }
     }
 }
 
+/// Persistence key for selected tab
+#[derive(Debug, Default, persisted::PersistedKey, Serialize)]
+#[persisted(Tab)]
+struct RecipeTabKey;
+
 #[derive(
     Copy,
     Clone,
