@@ -4,31 +4,26 @@ use crate::{
     context::TuiContext,
     message::Message,
     view::{
-        common::{
-            actions::{IntoMenuAction, MenuAction},
-            header_table::HeaderTable,
-        },
+        Component, ViewContext,
+        common::header_table::HeaderTable,
         component::queryable_body::QueryableBody,
         context::UpdateContext,
         draw::{Draw, DrawMetadata, Generate},
-        event::{Child, Emitter, Event, EventHandler, OptionEvent},
+        event::{Child, Event, EventHandler, OptionEvent},
         util::{persistence::PersistedLazy, view_text},
-        Component, ViewContext,
     },
 };
-use derive_more::Display;
+use mime::Mime;
 use persisted::PersistedKey;
 use ratatui::Frame;
-use serde::Serialize;
+use serde::{Serialize, Serializer};
 use slumber_config::Action;
 use slumber_core::{collection::RecipeId, http::ResponseRecord};
 use std::sync::Arc;
-use strum::{EnumIter, IntoEnumIterator};
 
 /// Display response body
 #[derive(Debug)]
 pub struct ResponseBodyView {
-    actions_emitter: Emitter<ResponseBodyMenuAction>,
     response: Arc<ResponseRecord>,
     /// The presentable version of the response body, which may or may not
     /// match the response body. We apply transformations such as filter,
@@ -40,61 +35,46 @@ impl ResponseBodyView {
     pub fn new(recipe_id: RecipeId, response: Arc<ResponseRecord>) -> Self {
         // Select default query based on content type
         let config = &TuiContext::get().config.commands;
-        let default_query = response
-            .mime()
-            .and_then(|mime| config.default_query.get(&mime).cloned());
+        let mime = response.mime();
+        let default_query = mime
+            .as_ref()
+            .and_then(|mime| config.default_query.get(mime).cloned());
         let body = PersistedLazy::new(
-            ResponseQueryKey(recipe_id),
+            ResponseQueryKey { recipe_id, mime },
             QueryableBody::new(Arc::clone(&response), default_query),
         )
         .into();
-        Self {
-            actions_emitter: Default::default(),
-            response,
-            body,
-        }
+        Self { response, body }
     }
 
-    fn view_body(&self) {
+    pub fn view_body(&self) {
         view_text(self.body.data().visible_text(), self.response.mime());
     }
+
+    pub fn copy_body(&self) {
+        // Use whatever text is visible to the user. This differs from saving
+        // the body, because we can't copy binary content, so if the file is
+        // binary we'll copy the hexcode text
+        ViewContext::send_message(Message::CopyText(
+            self.body.data().visible_text().to_string(),
+        ));
+    }
+
+    pub fn save_response_body(&self) {
+        // This will trigger a modal to ask the user for a path
+        ViewContext::send_message(Message::SaveResponseBody {
+            request_id: self.response.id,
+            data: self.body.data().modified_text(),
+        });
+    }
 }
 
 impl EventHandler for ResponseBodyView {
     fn update(&mut self, _: &mut UpdateContext, event: Event) -> Option<Event> {
-        event
-            .opt()
-            .action(|action, propagate| match action {
-                Action::View => self.view_body(),
-                _ => propagate.set(),
-            })
-            .emitted(self.actions_emitter, |menu_action| {
-                match menu_action {
-                    ResponseBodyMenuAction::ViewBody => self.view_body(),
-                    ResponseBodyMenuAction::CopyBody => {
-                        // Use whatever text is visible to the user. This
-                        // differs from saving the body, because we can't copy
-                        // binary content, so if the file is binary we'll copy
-                        // the hexcode text
-                        ViewContext::send_message(Message::CopyText(
-                            self.body.data().visible_text().to_string(),
-                        ));
-                    }
-                    ResponseBodyMenuAction::SaveBody => {
-                        // This will trigger a modal to ask the user for a path
-                        ViewContext::send_message(Message::SaveResponseBody {
-                            request_id: self.response.id,
-                            data: self.body.data().modified_text(),
-                        });
-                    }
-                }
-            })
-    }
-
-    fn menu_actions(&self) -> Vec<MenuAction> {
-        ResponseBodyMenuAction::iter()
-            .map(MenuAction::with_data(self, self.actions_emitter))
-            .collect()
+        event.opt().action(|action, propagate| match action {
+            Action::View => self.view_body(),
+            _ => propagate.set(),
+        })
     }
 
     fn children(&mut self) -> Vec<Component<Child<'_>>> {
@@ -103,36 +83,35 @@ impl EventHandler for ResponseBodyView {
 }
 
 impl Draw for ResponseBodyView {
-    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
+    fn draw(&self, frame: &mut Frame, (): (), metadata: DrawMetadata) {
         self.body.draw(frame, (), metadata.area(), true);
     }
 }
 
-/// Items in the actions popup menu for the Body
-#[derive(Copy, Clone, Debug, Display, EnumIter)]
-#[allow(clippy::enum_variant_names)]
-enum ResponseBodyMenuAction {
-    #[display("View Body")]
-    ViewBody,
-    #[display("Copy Body")]
-    CopyBody,
-    #[display("Save Body as File")]
-    SaveBody,
-}
-
-impl IntoMenuAction<ResponseBodyView> for ResponseBodyMenuAction {
-    fn shortcut(&self, _: &ResponseBodyView) -> Option<Action> {
-        match self {
-            Self::ViewBody => Some(Action::View),
-            Self::CopyBody | Self::SaveBody => None,
-        }
-    }
-}
-
 /// Persisted key for response body JSONPath query text box
 #[derive(Debug, Serialize, PersistedKey)]
 #[persisted(String)]
-struct ResponseQueryKey(RecipeId);
+struct ResponseQueryKey {
+    recipe_id: RecipeId,
+    /// Response queries are unique per-mime because query tools tend to be
+    /// specific to content type. If the content type changes (e.g. JSON
+    /// replaced by an HTML error page), we shouldn't keep applying JSON
+    /// commands
+    #[serde(serialize_with = "serialize_mime")]
+    mime: Option<Mime>,
+}
+
+/// Serialize a MIME type as its string representation
+#[expect(clippy::ref_option)]
+fn serialize_mime<S>(
+    mime: &Option<Mime>,
+    serializer: S,
+) -> Result<S::Ok, S::Error>
+where
+    S: Serializer,
+{
+    mime.as_ref().map(Mime::as_ref).serialize(serializer)
+}
 
 #[derive(Debug)]
 pub struct ResponseHeadersView {
@@ -148,14 +127,14 @@ impl ResponseHeadersView {
 impl EventHandler for ResponseHeadersView {}
 
 impl Draw for ResponseHeadersView {
-    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
+    fn draw(&self, frame: &mut Frame, (): (), metadata: DrawMetadata) {
         frame.render_widget(
             HeaderTable {
                 headers: &self.response.headers,
             }
             .generate(),
             metadata.area(),
-        )
+        );
     }
 }
 
@@ -163,17 +142,14 @@ impl Draw for ResponseHeadersView {
 mod tests {
     use super::*;
     use crate::{
-        test_util::{harness, terminal, TestHarness, TestTerminal},
+        test_util::{TestHarness, TestTerminal, harness, terminal},
         view::test_util::TestComponent,
     };
     use crossterm::event::KeyCode;
     use indexmap::indexmap;
     use rstest::rstest;
-    use slumber_core::{
-        assert_matches,
-        http::Exchange,
-        test_util::{header_map, Factory},
-    };
+    use slumber_core::{http::Exchange, test_util::header_map};
+    use slumber_util::{Factory, assert_matches};
 
     /// Test "Copy Body" menu action
     #[rstest]
@@ -210,7 +186,7 @@ mod tests {
             response: response.into(),
             ..Exchange::factory(())
         };
-        let mut component = TestComponent::new(
+        let component = TestComponent::new(
             &harness,
             &terminal,
             ResponseBodyView::new(
@@ -219,14 +195,7 @@ mod tests {
             ),
         );
 
-        // Open actions modal and select the copy action
-        component
-            .int()
-            .open_actions()
-            // Note: Edit Collections action isn't visible here
-            .send_keys([KeyCode::Down, KeyCode::Enter])
-            .assert_empty();
-
+        component.data().copy_body();
         let body = assert_matches!(
             harness.pop_message_now(),
             Message::CopyText(body) => body,
@@ -309,18 +278,11 @@ mod tests {
             })
             .await;
             // Background task sends a message to redraw
-            assert_matches!(harness.pop_message_now(), Message::Tick);
+            assert_matches!(harness.pop_message_now(), Message::Draw);
             component.int().drain_draw().assert_empty();
         }
 
-        // Open actions modal and select the save action
-        component
-            .int()
-            .open_actions()
-            // Note: Edit Collections action isn't visible here
-            .send_keys([KeyCode::Down, KeyCode::Down, KeyCode::Enter])
-            .assert_empty();
-
+        component.data().save_response_body();
         let (request_id, data) = assert_matches!(
             harness.pop_message_now(),
             Message::SaveResponseBody { request_id, data } => (request_id, data),
