@@ -1,8 +1,9 @@
 use crate::{
-    http::{RequestState, RequestStateSummary, RequestStore},
+    http::{RequestState, RequestStore},
     message::{Message, RequestConfig},
     util::ResultReported,
     view::{
+        Component, ViewContext,
         common::{
             actions::ActionsModal,
             modal::{Modal, ModalQueue},
@@ -11,24 +12,24 @@ use crate::{
             help::HelpFooter,
             history::History,
             misc::{ConfirmModal, NotificationText},
-            primary::PrimaryView,
+            primary::{PrimaryView, PrimaryViewProps},
         },
         context::UpdateContext,
         draw::{Draw, DrawMetadata, Generate},
         event::{Child, Event, EventHandler, OptionEvent},
         util::persistence::PersistedLazy,
-        Component, ViewContext,
     },
 };
 use derive_more::From;
 use persisted::{PersistedContainer, PersistedKey};
-use ratatui::{layout::Layout, prelude::Constraint, Frame};
+use ratatui::{Frame, layout::Layout, prelude::Constraint};
 use serde::Serialize;
 use slumber_config::Action;
 use slumber_core::{
     collection::{Collection, ProfileId},
     http::RequestId,
 };
+use std::ops::Deref;
 
 /// The root view component
 #[derive(Debug)]
@@ -44,14 +45,12 @@ pub struct Root {
 }
 
 impl Root {
-    pub fn new(collection: &Collection, request_store: &RequestStore) -> Self {
+    pub fn new(collection: &Collection) -> Self {
         // Load the selected request *second*, so it will take precedence over
         // the event that attempts to load the latest request for the recipe
         let selected_request_id: PersistedLazy<_, SelectedRequestId> =
             PersistedLazy::new_default(SelectedRequestKey);
-        let selected_request =
-            selected_request_id.0.and_then(|id| request_store.get(id));
-        let primary_view = PrimaryView::new(collection, selected_request);
+        let primary_view = PrimaryView::new(collection);
         Self {
             // State
             selected_request_id,
@@ -113,12 +112,13 @@ impl Root {
             // initialization where the recipe list asks for the latest
             // request *after* the selected ID is loaded from persistence
             let selected_request = self.selected_request(request_store);
-            if selected_request
-                .is_some_and(|request| request.recipe_id() == recipe_id)
-            {
+            let profile_id = primary_view.selected_profile_id();
+            if selected_request.is_some_and(|request| {
+                request.recipe_id() == recipe_id
+                    && request.profile_id() == profile_id
+            }) {
                 selected_request
             } else {
-                let profile_id = primary_view.selected_profile_id();
                 request_store.load_latest(profile_id, recipe_id)?
             }
         } else {
@@ -127,22 +127,9 @@ impl Root {
 
         *self.selected_request_id.get_mut() =
             state.map(RequestState::id).into();
-        // Update view with the new request
-        self.primary_view.data_mut().set_request_state(state);
         Ok(())
     }
 
-    /// Notify the component that a request's state has changed in the store. If
-    /// the request is selected, view state will be updated accordingly
-    pub fn update_request(&mut self, request_state: &RequestState) {
-        let selected_request_id = self.selected_request_id();
-        if selected_request_id == Some(request_state.id()) {
-            self.primary_view
-                .data_mut()
-                .set_request_state(Some(request_state));
-        }
-    }
-
     /// Open the history modal for current recipe+profile. Return an error if
     /// the harness.database load failed.
     fn open_history(
@@ -154,7 +141,6 @@ impl Root {
             // Make sure all requests for this profile+recipe are loaded
             let requests = request_store
                 .load_summaries(primary_view.selected_profile_id(), recipe_id)?
-                .map(RequestStateSummary::from)
                 .collect();
 
             History::new(recipe_id, requests, self.selected_request_id())
@@ -185,8 +171,9 @@ impl EventHandler for Root {
                 }
                 Action::Cancel => {
                     if let Some(request_id) = self.selected_request_id.0 {
-                        // 2024 edition: if-let chain
-                        if context.request_store.is_in_progress(request_id) {
+                        // unstable: if-let chain
+                        // https://github.com/rust-lang/rust/pull/132833
+                        if context.request_store.can_cancel(request_id) {
                             ConfirmModal::new(
                                 "Cancel request?".into(),
                                 move |response| {
@@ -197,13 +184,13 @@ impl EventHandler for Root {
                                     }
                                 },
                             )
-                            .open()
+                            .open();
                         }
                     }
                 }
                 Action::Quit => ViewContext::send_message(Message::Quit),
                 Action::ReloadCollection => {
-                    ViewContext::send_message(Message::CollectionStartReload)
+                    ViewContext::send_message(Message::CollectionStartReload);
                 }
                 _ => propagate.set(),
             })
@@ -239,17 +226,26 @@ impl EventHandler for Root {
     }
 }
 
-impl Draw for Root {
-    fn draw(&self, frame: &mut Frame, _: (), metadata: DrawMetadata) {
+impl<R: Deref<Target = RequestStore>> Draw<RootProps<R>> for Root {
+    fn draw(
+        &self,
+        frame: &mut Frame,
+        props: RootProps<R>,
+        metadata: DrawMetadata,
+    ) {
         // Create layout
         let [main_area, footer_area] =
             Layout::vertical([Constraint::Min(0), Constraint::Length(1)])
                 .areas(metadata.area());
 
         // Main content
+        let selected_request = self
+            .selected_request_id
+            .0
+            .and_then(|id| props.request_store.get(id));
         self.primary_view.draw(
             frame,
-            (),
+            PrimaryViewProps { selected_request },
             main_area,
             !self.modal_queue.data().is_open(),
         );
@@ -270,6 +266,11 @@ impl Draw for Root {
     }
 }
 
+#[derive(Debug)]
+pub struct RootProps<R: Deref<Target = RequestStore>> {
+    pub request_store: R,
+}
+
 /// Persistence key for the selected request
 #[derive(Debug, Serialize, PersistedKey)]
 #[persisted(Option<RequestId>)]
@@ -291,7 +292,7 @@ impl PersistedContainer for SelectedRequestId {
     fn restore_persisted(&mut self, request_id: Self::Value) {
         // We can't just set the value directly, because then the request won't
         // be loaded from the DB
-        ViewContext::push_event(Event::HttpSelectRequest(request_id))
+        ViewContext::push_event(Event::HttpSelectRequest(request_id));
     }
 }
 
@@ -299,34 +300,46 @@ impl PersistedContainer for SelectedRequestId {
 mod tests {
     use super::*;
     use crate::{
-        test_util::{harness, terminal, TestHarness, TestTerminal},
+        test_util::{TestHarness, TestTerminal, harness, terminal},
         view::{
             test_util::TestComponent, util::persistence::DatabasePersistedStore,
         },
     };
+    use crossterm::event::KeyCode;
     use persisted::PersistedStore;
     use rstest::rstest;
-    use slumber_core::{http::Exchange, test_util::Factory};
+    use slumber_core::{
+        collection::{Profile, Recipe},
+        http::Exchange,
+        test_util::by_id,
+    };
+    use slumber_util::Factory;
 
     /// Test that, on first render, the view loads the most recent historical
     /// request for the first recipe+profile
     #[rstest]
     fn test_preload_request(harness: TestHarness, terminal: TestTerminal) {
         // Add a request into the DB that we expect to preload
-
-        let collection = Collection::factory(());
-        let profile_id = collection.first_profile_id();
-        let recipe_id = collection.first_recipe_id();
+        let profile_id = harness.collection.first_profile_id();
+        let recipe_id = harness.collection.first_recipe_id();
         let exchange =
             Exchange::factory((Some(profile_id.clone()), recipe_id.clone()));
         harness.database.insert_exchange(&exchange).unwrap();
 
-        let mut component = TestComponent::new(
+        let props_factory = || RootProps {
+            request_store: harness.request_store.borrow(),
+        };
+        let mut component = TestComponent::builder(
             &harness,
             &terminal,
-            Root::new(&collection, &harness.request_store.borrow()),
-        );
-        component.int().drain_draw().assert_empty();
+            Root::new(&harness.collection),
+        )
+        .with_props(props_factory())
+        .build();
+        component
+            .int_props(props_factory)
+            .drain_draw()
+            .assert_empty();
 
         // Make sure profile+recipe were preselected correctly
         let primary_view = component.data().primary_view.data();
@@ -345,9 +358,8 @@ mod tests {
         harness: TestHarness,
         terminal: TestTerminal,
     ) {
-        let collection = Collection::factory(());
-        let recipe_id = collection.first_recipe_id();
-        let profile_id = collection.first_profile_id();
+        let recipe_id = harness.collection.first_recipe_id();
+        let profile_id = harness.collection.first_profile_id();
         // This is the older one, but it should be loaded because of persistence
         let old_exchange =
             Exchange::factory((Some(profile_id.clone()), recipe_id.clone()));
@@ -360,12 +372,20 @@ mod tests {
             &Some(old_exchange.id),
         );
 
-        let mut component = TestComponent::new(
+        let props_factory = || RootProps {
+            request_store: harness.request_store.borrow(),
+        };
+        let mut component = TestComponent::builder(
             &harness,
             &terminal,
-            Root::new(&collection, &harness.request_store.borrow()),
-        );
-        component.int().drain_draw().assert_empty();
+            Root::new(&harness.collection),
+        )
+        .with_props(props_factory())
+        .build();
+        component
+            .int_props(props_factory)
+            .drain_draw()
+            .assert_empty();
 
         // Make sure everything was preselected correctly
         assert_eq!(
@@ -389,9 +409,8 @@ mod tests {
         harness: TestHarness,
         terminal: TestTerminal,
     ) {
-        let collection = Collection::factory(());
-        let recipe_id = collection.first_recipe_id();
-        let profile_id = collection.first_profile_id();
+        let recipe_id = harness.collection.first_recipe_id();
+        let profile_id = harness.collection.first_profile_id();
         let old_exchange =
             Exchange::factory((Some(profile_id.clone()), recipe_id.clone()));
         let new_exchange =
@@ -407,16 +426,272 @@ mod tests {
             )
             .unwrap();
 
-        let mut component = TestComponent::new(
+        let props_factory = || RootProps {
+            request_store: harness.request_store.borrow(),
+        };
+        let mut component = TestComponent::builder(
             &harness,
             &terminal,
-            Root::new(&collection, &harness.request_store.borrow()),
-        );
-        component.int().drain_draw().assert_empty();
+            Root::new(&harness.collection),
+        )
+        .with_props(props_factory())
+        .build();
+        component
+            .int_props(props_factory)
+            .drain_draw()
+            .assert_empty();
 
         assert_eq!(
             component.data().selected_request_id(),
             Some(new_exchange.id)
         );
     }
+
+    /// Test that when the selected recipe changes, the selected request changes
+    /// as well
+    #[rstest]
+    fn test_recipe_change(terminal: TestTerminal) {
+        let recipe1 = Recipe::factory(());
+        let recipe2 = Recipe::factory(());
+        let recipe1_id = recipe1.id.clone();
+        let recipe2_id = recipe2.id.clone();
+        let collection = Collection {
+            recipes: by_id([recipe1, recipe2]).into(),
+            ..Collection::factory(())
+        };
+        let harness = TestHarness::new(collection);
+        let profile_id = harness.collection.first_profile_id();
+        let exchange1 =
+            Exchange::factory((Some(profile_id.clone()), recipe1_id.clone()));
+        let exchange2 =
+            Exchange::factory((Some(profile_id.clone()), recipe2_id.clone()));
+        harness.database.insert_exchange(&exchange1).unwrap();
+        harness.database.insert_exchange(&exchange2).unwrap();
+
+        let props_factory = || RootProps {
+            request_store: harness.request_store.borrow(),
+        };
+        let mut component = TestComponent::builder(
+            &harness,
+            &terminal,
+            Root::new(&harness.collection),
+        )
+        .with_props(props_factory())
+        .build();
+        component
+            .int_props(props_factory)
+            .drain_draw()
+            .assert_empty();
+
+        assert_eq!(component.data().selected_request_id(), Some(exchange1.id));
+
+        // Select the second recipe
+        component
+            .int_props(props_factory)
+            .send_keys([KeyCode::Char('l'), KeyCode::Down])
+            .assert_empty();
+        assert_eq!(component.data().selected_request_id(), Some(exchange2.id));
+    }
+
+    /// Test that when the selected profile changes, the selected request
+    /// changes as well
+    #[rstest]
+    fn test_profile_change(terminal: TestTerminal) {
+        let profile1 = Profile::factory(());
+        let profile2 = Profile::factory(());
+        let profile1_id = profile1.id.clone();
+        let profile2_id = profile2.id.clone();
+        let collection = Collection {
+            profiles: by_id([profile1, profile2]),
+            ..Collection::factory(())
+        };
+        let harness = TestHarness::new(collection);
+        let recipe_id = harness.collection.first_recipe_id();
+        let exchange1 =
+            Exchange::factory((Some(profile1_id.clone()), recipe_id.clone()));
+        let exchange2 =
+            Exchange::factory((Some(profile2_id.clone()), recipe_id.clone()));
+        harness.database.insert_exchange(&exchange1).unwrap();
+        harness.database.insert_exchange(&exchange2).unwrap();
+
+        let props_factory = || RootProps {
+            request_store: harness.request_store.borrow(),
+        };
+        let mut component = TestComponent::builder(
+            &harness,
+            &terminal,
+            Root::new(&harness.collection),
+        )
+        .with_props(props_factory())
+        .build();
+        component
+            .int_props(props_factory)
+            .drain_draw()
+            .assert_empty();
+
+        assert_eq!(component.data().selected_request_id(), Some(exchange1.id));
+
+        // Select the second profile
+        component
+            .int_props(props_factory)
+            .send_keys([KeyCode::Char('p'), KeyCode::Down, KeyCode::Enter])
+            .assert_empty();
+        assert_eq!(component.data().selected_request_id(), Some(exchange2.id));
+    }
+
+    /// Test "Delete Requests" action via both the recipe pane
+    #[rstest]
+    fn test_delete_recipe_requests(
+        harness: TestHarness,
+        #[with(80, 20)] terminal: TestTerminal,
+    ) {
+        let recipe_id = harness.collection.first_recipe_id();
+        let profile_id = harness.collection.first_profile_id();
+        let old_exchange =
+            Exchange::factory((Some(profile_id.clone()), recipe_id.clone()));
+        let new_exchange =
+            Exchange::factory((Some(profile_id.clone()), recipe_id.clone()));
+        harness.database.insert_exchange(&old_exchange).unwrap();
+        harness.database.insert_exchange(&new_exchange).unwrap();
+
+        let props_factory = || RootProps {
+            request_store: harness.request_store.borrow(),
+        };
+        let mut component = TestComponent::builder(
+            &harness,
+            &terminal,
+            Root::new(&harness.collection),
+        )
+        .with_props(props_factory())
+        .build();
+        // Select recipe pane
+        component
+            .int_props(props_factory)
+            .drain_draw()
+            .send_key(KeyCode::Char('c'))
+            .assert_empty();
+
+        // Sanity check for initial state
+        assert_eq!(
+            component.data().selected_request_id(),
+            Some(new_exchange.id)
+        );
+
+        // Select "Delete Requests" but decline the confirmation
+        component
+            .int_props(props_factory)
+            .open_actions()
+            .send_keys([
+                // "Delete Request" action
+                KeyCode::Down,
+                KeyCode::Down,
+                KeyCode::Down,
+                KeyCode::Enter,
+                // Decline
+                KeyCode::Left,
+                KeyCode::Enter,
+            ])
+            .assert_empty();
+
+        // Same request is still selected
+        assert_eq!(
+            component.data().selected_request_id(),
+            Some(new_exchange.id)
+        );
+
+        // Select "Delete Requests" and accept. I don't feel like testing Delete
+        // for All Profiles
+        component
+            .int_props(props_factory)
+            .open_actions()
+            .send_keys([
+                // "Delete Request" action
+                KeyCode::Down,
+                KeyCode::Down,
+                KeyCode::Down,
+                KeyCode::Enter,
+                // Confirm
+                KeyCode::Enter,
+            ])
+            .assert_empty();
+
+        assert_eq!(component.data().selected_request_id(), None);
+    }
+
+    /// Test "Delete Request" action, which is available via the
+    /// Request/Response pane
+    #[rstest]
+    fn test_delete_request(harness: TestHarness, terminal: TestTerminal) {
+        let recipe_id = harness.collection.first_recipe_id();
+        let profile_id = harness.collection.first_profile_id();
+        let old_exchange =
+            Exchange::factory((Some(profile_id.clone()), recipe_id.clone()));
+        let new_exchange =
+            Exchange::factory((Some(profile_id.clone()), recipe_id.clone()));
+        harness.database.insert_exchange(&old_exchange).unwrap();
+        harness.database.insert_exchange(&new_exchange).unwrap();
+
+        let props_factory = || RootProps {
+            request_store: harness.request_store.borrow(),
+        };
+        let mut component = TestComponent::builder(
+            &harness,
+            &terminal,
+            Root::new(&harness.collection),
+        )
+        .with_props(props_factory())
+        .build();
+        // Select exchange pane
+        component
+            .int_props(props_factory)
+            .drain_draw()
+            .send_key(KeyCode::Char('r'))
+            .assert_empty();
+
+        // Sanity check for initial state
+        assert_eq!(
+            component.data().selected_request_id(),
+            Some(new_exchange.id)
+        );
+
+        // Select "Delete Request" but decline the confirmation
+        component
+            .int_props(props_factory)
+            .open_actions()
+            .send_keys([
+                // "Delete Request" action
+                KeyCode::Up,
+                KeyCode::Enter,
+                // Decline
+                KeyCode::Left,
+                KeyCode::Enter,
+            ])
+            .assert_empty();
+
+        // Same request is still selected
+        assert_eq!(
+            component.data().selected_request_id(),
+            Some(new_exchange.id)
+        );
+
+        component
+            .int_props(props_factory)
+            .send_key(KeyCode::Char('r'))
+            .open_actions()
+            .send_keys([
+                // "Delete Request" action
+                KeyCode::Up,
+                KeyCode::Enter,
+                // Confirm
+                KeyCode::Enter,
+            ])
+            .assert_empty();
+
+        // New exchange is gone
+        assert_eq!(
+            component.data().selected_request_id(),
+            Some(old_exchange.id)
+        );
+        assert_eq!(harness.request_store.borrow().get(new_exchange.id), None);
+    }
 }
