@@ -4,10 +4,10 @@
 use crate::{
     util::Flag,
     view::{
+        Component, ViewContext,
         common::{actions::MenuAction, modal::Modal},
         context::UpdateContext,
         state::Notification,
-        Component, ViewContext,
     },
 };
 use persisted::{PersistedContainer, PersistedLazyRefMut, PersistedStore};
@@ -104,7 +104,7 @@ impl<T: EventHandler> EventHandler for Option<T> {
 // PersistedLazy's custom ToChild impl, which interferes with the blanket
 // ToChild impl
 
-impl<'a> EventHandler for Child<'a> {
+impl EventHandler for Child<'_> {
     fn update(
         &mut self,
         context: &mut UpdateContext,
@@ -118,7 +118,7 @@ impl<'a> EventHandler for Child<'a> {
     }
 }
 
-impl<'a, S, K, C> EventHandler for PersistedLazyRefMut<'a, S, K, C>
+impl<S, K, C> EventHandler for PersistedLazyRefMut<'_, S, K, C>
 where
     S: PersistedStore<K>,
     K: persisted::PersistedKey,
@@ -159,7 +159,7 @@ impl<'a> Deref for Child<'a> {
     }
 }
 
-impl<'a> DerefMut for Child<'a> {
+impl DerefMut for Child<'_> {
     fn deref_mut(&mut self) -> &mut Self::Target {
         match self {
             Child::Borrowed(inner) => *inner,
@@ -279,6 +279,7 @@ pub enum Event {
 impl Event {
     /// Convert to `Option<Event>` so the methods from [OptionEvent] can be used
     /// to match the event
+    #[expect(clippy::unnecessary_wraps)]
     pub fn opt(self) -> Option<Event> {
         Some(self)
     }
@@ -324,11 +325,7 @@ impl OptionEvent for Option<Event> {
         {
             let mut propagate = Flag::default();
             f(*action, &mut propagate);
-            if *propagate {
-                Some(event)
-            } else {
-                None
-            }
+            if *propagate { Some(event) } else { None }
         } else {
             Some(event)
         }
@@ -354,25 +351,9 @@ impl OptionEvent for Option<Event> {
 /// A wrapper trait for [Any] that also gives us access to the type's [Debug]
 /// impl. This makes testing and logging much more effective, because we get the
 /// value's underlying debug representation, rather than just `Any {..}`.
-pub trait LocalEvent: Any + Debug {
-    // Workaround for trait upcasting
-    // unstable: Delete this once we get trait upcasting
-    // https://github.com/rust-lang/rust/issues/65991
-    fn into_any(self: Box<Self>) -> Box<dyn Any>;
-}
+pub trait LocalEvent: Any + Debug {}
 
-impl<T: Any + Debug> LocalEvent for T {
-    fn into_any(self: Box<Self>) -> Box<dyn Any> {
-        self
-    }
-}
-
-impl dyn LocalEvent {
-    /// Alias for `Any::downcast`, to downcast into a concrete type
-    pub fn downcast<T: Any>(self: Box<dyn LocalEvent>) -> Option<T> {
-        self.into_any().downcast().map(|b| *b).ok()
-    }
-}
+impl<T: Any + Debug> LocalEvent for T {}
 
 /// An emitter generates events of a particular type. This is used for
 /// components that need to respond to actions performed on their children, e.g.
@@ -430,14 +411,16 @@ impl<T: Sized + LocalEvent> Emitter<T> {
             } if emitter_id == self.id => {
                 // This cast should be infallible because emitter IDs are unique
                 // and each emitter can only emit one type
-                Ok(event.downcast::<T>().unwrap_or_else(|| {
-                    panic!(
-                        "Incorrect emitted event type for emitter \
+                Ok(*(event as Box<dyn Any>).downcast::<T>().unwrap_or_else(
+                    |_| {
+                        panic!(
+                            "Incorrect emitted event type for emitter \
                         `{emitter_id}`. Expected type {}, received type \
                         {emitter_type}",
-                        any::type_name::<T>()
-                    )
-                }))
+                            any::type_name::<T>()
+                        )
+                    },
+                ))
             }
             _ => Err(event),
         }
