@@ -5,8 +5,8 @@ use crate::view::{
 };
 use persisted::PersistedContainer;
 use ratatui::{
-    widgets::{ListState, StatefulWidget, TableState},
     Frame,
+    widgets::{ListState, StatefulWidget, TableState},
 };
 use slumber_config::Action;
 use slumber_core::collection::HasId;
@@ -218,7 +218,7 @@ impl<Item, State: SelectStateData> SelectState<Item, State> {
     pub fn select_index(&mut self, index: usize) {
         let state = self.state.get_mut();
         let current = state.selected();
-        state.select(index);
+        state.select(Some(index));
         let new = state.selected();
 
         // If the selection changed, send an event
@@ -237,6 +237,33 @@ impl<Item, State: SelectStateData> SelectState<Item, State> {
         self.select_delta(1);
     }
 
+    /// Remove the selected item from the list. This will slide everything after
+    /// that item up one slot, so that the item after it is selected. If the
+    /// selected item was at the end of the list, the item before it will be
+    /// selected. If the list is now empty, the selection is cleared.
+    pub fn delete_selected(&mut self) {
+        let state = self.state.get_mut();
+        let selected_index = state.selected();
+        if let Some(index) = selected_index {
+            self.items.remove(index);
+
+            // There are two ways the selection could now be invalid:
+            // - Deleted the only item in the list. Clear the selection
+            // - Deleted the item at the end of the list. Select the item before
+            //   it
+            // Otherwise, we implicitly selected the item after the deleted one,
+            // and can proceed as usual
+            if self.items.is_empty() {
+                state.select(None);
+            } else if index == self.items.len() {
+                state.select(Some(index - 1));
+            }
+
+            // This will do nothing if the selection is now empty
+            self.emit_for_selected(SelectStateEvent::Select);
+        }
+    }
+
     /// Move some number of items up or down the list. Selection will wrap if
     /// it underflows/overflows.
     fn select_delta(&mut self, delta: isize) {
@@ -259,7 +286,8 @@ impl<Item, State: SelectStateData> SelectState<Item, State> {
     /// event will *not* be emitted if no item is select, or the selected item
     /// is disabled.
     fn emit_for_selected(&self, event_fn: impl Fn(usize) -> SelectStateEvent) {
-        // 2024 edition: if-let chain
+        // unstable: if-let chain
+        // https://github.com/rust-lang/rust/pull/132833
         match self.selected_index() {
             // Don't send event for disabled items
             Some(selected) if !self.items[selected].disabled => {
@@ -326,12 +354,12 @@ where
             Action::Toggle
                 if self.is_subscribed(SelectStateEventType::Toggle) =>
             {
-                self.emit_for_selected(SelectStateEvent::Toggle)
+                self.emit_for_selected(SelectStateEvent::Toggle);
             }
             Action::Submit
                 if self.is_subscribed(SelectStateEventType::Submit) =>
             {
-                self.emit_for_selected(SelectStateEvent::Submit)
+                self.emit_for_selected(SelectStateEvent::Submit);
             }
             _ => propagate.set(),
         })
@@ -395,8 +423,8 @@ pub trait SelectStateData: Default {
     /// Index of the selected element
     fn selected(&self) -> Option<usize>;
 
-    /// Select an element by index
-    fn select(&mut self, index: usize);
+    /// Select an element by index. Clear the selection if `None` is given
+    fn select(&mut self, index: Option<usize>);
 }
 
 impl SelectStateData for ListState {
@@ -404,8 +432,8 @@ impl SelectStateData for ListState {
         self.selected()
     }
 
-    fn select(&mut self, index: usize) {
-        self.select(Some(index))
+    fn select(&mut self, index: Option<usize>) {
+        self.select(index);
     }
 }
 
@@ -414,18 +442,20 @@ impl SelectStateData for TableState {
         self.selected()
     }
 
-    fn select(&mut self, index: usize) {
-        self.select(Some(index))
+    fn select(&mut self, index: Option<usize>) {
+        self.select(index);
     }
 }
 
+/// Selection state is just a number. This is useful for tabs and other
+/// fixed-size elements that can never be empty.
 impl SelectStateData for usize {
     fn selected(&self) -> Option<usize> {
         Some(*self)
     }
 
-    fn select(&mut self, index: usize) {
-        *self = index;
+    fn select(&mut self, index: Option<usize>) {
+        *self = index.expect("Cannot clear selection");
     }
 }
 
@@ -454,7 +484,7 @@ where
 mod tests {
     use super::*;
     use crate::{
-        test_util::{harness, terminal, TestHarness, TestTerminal},
+        test_util::{TestHarness, TestTerminal, harness, terminal},
         view::{
             test_util::TestComponent,
             util::persistence::{DatabasePersistedStore, PersistedLazy},
@@ -465,57 +495,128 @@ mod tests {
     use ratatui::widgets::List;
     use rstest::{fixture, rstest};
     use serde::Serialize;
-    use slumber_core::{assert_matches, collection::ProfileId};
+    use slumber_core::collection::ProfileId;
+    use slumber_util::assert_matches;
 
     /// Test going up and down in the list
     #[rstest]
     fn test_navigation(
         harness: TestHarness,
         terminal: TestTerminal,
-        items: (Vec<&'static str>, List<'static>),
+        #[from(items)] (items, props): (Vec<&'static str>, List<'static>),
     ) {
-        let select = SelectState::builder(items.0).build();
+        let select = SelectState::builder(items).build();
         let mut component = TestComponent::builder(&harness, &terminal, select)
-            .with_props(items.1)
+            .with_props(props.clone())
             .build();
-        component.int().drain_draw().assert_empty();
+        component
+            .int_props(|| props.clone())
+            .drain_draw()
+            .assert_empty();
         assert_eq!(component.data().selected(), Some(&"a"));
-        component.int().send_key(KeyCode::Down).assert_empty();
+        component
+            .int_props(|| props.clone())
+            .send_key(KeyCode::Down)
+            .assert_empty();
         assert_eq!(component.data().selected(), Some(&"b"));
 
-        component.int().send_key(KeyCode::Up).assert_empty();
+        component
+            .int_props(|| props.clone())
+            .send_key(KeyCode::Up)
+            .assert_empty();
         assert_eq!(component.data().selected(), Some(&"a"));
     }
 
+    /// Test deleting the selected item
+    #[rstest]
+    fn test_delete(
+        harness: TestHarness,
+        terminal: TestTerminal,
+        #[from(items)] (items, props): (Vec<&'static str>, List<'static>),
+    ) {
+        let select = SelectState::builder(items)
+            .subscribe([SelectStateEventType::Select])
+            .build();
+        let mut component = TestComponent::builder(&harness, &terminal, select)
+            .with_props(props.clone())
+            .build();
+
+        // Start by selecting the second item, so we can assert that we select
+        // the one after it when possible
+        component
+            .int_props(|| props.clone())
+            .drain_draw() // Handle  initial state
+            .send_key(KeyCode::Down)
+            .assert_emitted([
+                SelectStateEvent::Select(0),
+                SelectStateEvent::Select(1),
+            ]);
+        assert_eq!(component.data().selected(), Some(&"b"));
+
+        // Delete `b`, `c` should get selected because it's below
+        component.data_mut().delete_selected();
+        assert_eq!(component.data().selected(), Some(&"c"));
+        component
+            .int_props(|| props.clone())
+            .drain_draw()
+            .assert_emitted([SelectStateEvent::Select(1)]);
+
+        // Delete `c`; there's nothing left below so select above
+        component.data_mut().delete_selected();
+        assert_eq!(component.data().selected(), Some(&"a"));
+        component
+            .int_props(|| props.clone())
+            .drain_draw()
+            .assert_emitted([SelectStateEvent::Select(0)]);
+
+        // Delete `a`, nothing left to select
+        component.data_mut().delete_selected();
+        assert_eq!(component.data().selected(), None);
+        component
+            .int_props(|| props.clone())
+            .drain_draw()
+            .assert_emitted([]);
+
+        // Delete nothing; nothing should happen
+        component.data_mut().delete_selected();
+        component
+            .int_props(|| props.clone())
+            .drain_draw()
+            .assert_emitted([]);
+    }
+
     /// Test select emitted event
     #[rstest]
     fn test_select(
         harness: TestHarness,
         terminal: TestTerminal,
-        items: (Vec<&'static str>, List<'static>),
+        #[from(items)] (items, props): (Vec<&'static str>, List<'static>),
     ) {
-        let select = SelectState::builder(items.0)
+        let select = SelectState::builder(items)
             .disabled_indexes([2])
             .subscribe([SelectStateEventType::Select])
             .build();
         let mut component = TestComponent::builder(&harness, &terminal, select)
-            .with_props(items.1)
+            .with_props(props.clone())
             .build();
 
         // Initial selection
         assert_eq!(component.data().selected(), Some(&"a"));
         component
-            .int()
+            .int_props(|| props.clone())
             .drain_draw()
             .assert_emitted([SelectStateEvent::Select(0)]);
 
         component
-            .int()
+            .int_props(|| props.clone())
             .send_key(KeyCode::Down)
             .assert_emitted([SelectStateEvent::Select(1)]);
 
         // "c" is disabled, should not trigger events
-        component.int().send_key(KeyCode::Down).assert_empty();
+        component
+            .int_props(|| props.clone())
+            .send_key(KeyCode::Down)
+            .assert_empty();
     }
 
     /// Test submit emitted event
@@ -523,25 +624,28 @@ mod tests {
     fn test_submit(
         harness: TestHarness,
         terminal: TestTerminal,
-        items: (Vec<&'static str>, List<'static>),
+        #[from(items)] (items, props): (Vec<&'static str>, List<'static>),
     ) {
-        let select = SelectState::builder(items.0)
+        let select = SelectState::builder(items)
             .disabled_indexes([2])
             .subscribe([SelectStateEventType::Submit])
             .build();
         let mut component = TestComponent::builder(&harness, &terminal, select)
-            .with_props(items.1)
+            .with_props(props.clone())
             .build();
-        component.int().drain_draw().assert_empty();
+        component
+            .int_props(|| props.clone())
+            .drain_draw()
+            .assert_empty();
 
         component
-            .int()
+            .int_props(|| props.clone())
             .send_keys([KeyCode::Down, KeyCode::Enter])
             .assert_emitted([SelectStateEvent::Submit(1)]);
 
         // "c" is disabled, should not trigger events
         component
-            .int()
+            .int_props(|| props.clone())
             .send_keys([KeyCode::Down, KeyCode::Enter])
             .assert_empty();
     }
@@ -552,22 +656,28 @@ mod tests {
     fn test_propagate(
         harness: TestHarness,
         terminal: TestTerminal,
-        items: (Vec<&'static str>, List<'static>),
+        #[from(items)] (items, props): (Vec<&'static str>, List<'static>),
     ) {
-        let select = SelectState::builder(items.0).build();
+        let select = SelectState::builder(items).build();
         let mut component = TestComponent::builder(&harness, &terminal, select)
-            .with_props(items.1)
+            .with_props(props.clone())
             .build();
 
         assert_matches!(
-            component.int().send_key(KeyCode::Enter).events(),
+            component
+                .int_props(|| props.clone())
+                .send_key(KeyCode::Enter)
+                .events(),
             &[Event::Input {
                 action: Some(Action::Submit),
                 ..
             }]
         );
         assert_matches!(
-            component.int().send_key(KeyCode::Char(' ')).events(),
+            component
+                .int_props(|| props.clone())
+                .send_key(KeyCode::Char(' '))
+                .events(),
             &[Event::Input {
                 action: Some(Action::Toggle),
                 ..
@@ -625,17 +735,21 @@ mod tests {
             &terminal,
             PersistedLazy::new(Key, select),
         )
-        .with_props(list)
+        .with_props(list.clone())
         .build();
         assert_eq!(
             component.data().selected().map(ProfileItem::id),
             Some(&profile_id)
         );
-        component.int().drain_draw().assert_emitted([
-            // First item gets selected by preselection, second by persistence
-            SelectStateEvent::Select(0),
-            SelectStateEvent::Select(1),
-        ])
+        component
+            .int_props(|| list.clone())
+            .drain_draw()
+            .assert_emitted([
+                // First item gets selected by preselection, second by
+                // persistence
+                SelectStateEvent::Select(0),
+                SelectStateEvent::Select(1),
+            ]);
     }
 
     #[fixture]
